from plone.memoize import view
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.interfaces import INonInstallable
from Products.GenericSetup import EXTENSION
from Products.GenericSetup.tool import UNKNOWN
from zope.component import getAllUtilitiesRegisteredFor

import logging
import pkg_resources


logger = logging.getLogger("Plone")


class Addons:
    """Performs install/upgrade/uninstall functions on an addon.
    Pulled, mostly intact, from Plone 5.1's products control panel.
    If we reach the point when plone.restapi isn't supporting releases
    prior to 5.1, we might be able to remove this as duplicate code.
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request

        self.ps = getToolByName(context, "portal_setup")
        self.errors = {}

    def serializeAddon(self, addon):
        return {
            "@id": "{}/@addons/{}".format(self.context.absolute_url(), addon["id"]),
            "id": addon["id"],
            "title": addon["title"],
            "description": addon["description"],
            "install_profile_id": addon["install_profile_id"],
            "is_installed": addon["is_installed"],
            "profile_type": addon["profile_type"],
            "uninstall_profile_id": addon["uninstall_profile_id"],
            "version": addon["version"],
            "upgrade_info": addon["upgrade_info"],
        }

    def is_profile_installed(self, profile_id):
        return self.ps.getLastVersionForProfile(profile_id) != UNKNOWN

    def is_product_installed(self, product_id):
        profile = self.get_install_profile(product_id, allow_hidden=True)
        if not profile:
            return False
        return self.is_profile_installed(profile["id"])

    def _install_profile_info(self, product_id):
        """List extension profile infos of a given product."""
        profiles = self.ps.listProfileInfo()
        # We are only interested in extension profiles for the product.
        # TODO Remove the manual Products.* check here. It is still needed.
        profiles = [
            prof
            for prof in profiles
            if prof["type"] == EXTENSION
            and (
                prof["product"]
                in (
                    product_id,
                    f"Products.{product_id}",
                )
            )
        ]
        return profiles

    def _get_profile(self, product_id, name, strict=True, allow_hidden=False):
        """Return profile with given name.

        Also return None when no profiles are found at all.

        :param product_id: id of product/package.
            For example CMFPlone or plone.app.registry.
        :type product_id: string
        :param name: name of profile.
            Usually 'default' or 'uninstall'.
        :type name: string
        :param strict: When True, return None when name is not found.
            Otherwise fall back to the first profile.
        :type strict: boolean
        :param allow_hidden: Allow getting hidden profile.
            A non hidden profile is always preferred.
        :type allow_hidden: boolean
        :returns: True on success, False otherwise.
        :rtype: boolean
        """
        profiles = self._install_profile_info(product_id)
        if not profiles:
            return
        utils = getAllUtilitiesRegisteredFor(INonInstallable)
        hidden = []
        for util in utils:
            gnip = getattr(util, "getNonInstallableProfiles", None)
            if gnip is None:
                continue
            hidden.extend(gnip())

        # We have prime candidates that we prefer, and have hidden candidates
        # in case allow_hidden is True.
        prime_candidates = []
        hidden_candidates = []
        for profile in profiles:
            profile_id = profile["id"]
            profile_id_parts = profile_id.split(":")
            if len(profile_id_parts) != 2:
                logger.error("Profile with id '%s' is invalid." % profile_id)
                continue
            if allow_hidden and profile_id in hidden:
                if profile_id_parts[1] == name:
                    # This will especially be true for uninstall profiles,
                    # which are usually hidden.
                    return profile
                hidden_candidates.append(profile)
                continue
            if profile_id_parts[1] == name:
                return profile
            prime_candidates.append(profile)
        if strict:
            return
        if prime_candidates:
            # QI used to pick the first profile.
            # Return the first profile after all.
            return prime_candidates[0]
        if allow_hidden and hidden_candidates:
            # Return the first hidden profile.
            return hidden_candidates[0]

    def get_install_profile(self, product_id, allow_hidden=False):
        """Return the default install profile.

        :param product_id: id of product/package
        :type product_id: string
        :param allow_hidden: Allow getting otherwise hidden profile.
            In the UI this will be False, but you can set it to True in
            for example a call from plone.app.upgrade where you want to
            install a new core product, even though it is hidden for users.
        :type allow_hidden: boolean
        :returns: True on success, False otherwise.
        :rtype: boolean
        """
        return self._get_profile(
            product_id, "default", strict=False, allow_hidden=allow_hidden
        )

    def get_uninstall_profile(self, product_id):
        """Return the uninstall profile.

        Note: not used yet.
        """
        return self._get_profile(
            product_id, "uninstall", strict=True, allow_hidden=True
        )

    def is_product_installable(self, product_id, allow_hidden=False):
        """Does a product have an installation profile?

        :param allow_hidden: Allow installing otherwise hidden products.
            In the UI this will be False, but you can set it to True in
            for example a call from plone.app.upgrade where you want to
            install a new core product, even though it is hidden for users.
        :type allow_hidden: boolean
        :returns: True when product is installable, False otherwise.
        :rtype: boolean
        """
        if not allow_hidden:
            not_installable = []
            utils = getAllUtilitiesRegisteredFor(INonInstallable)
            for util in utils:
                gnip = getattr(util, "getNonInstallableProducts", None)
                if gnip is None:
                    continue
                not_installable.extend(gnip())
            if product_id in not_installable:
                return False

        profile = self.get_install_profile(product_id, allow_hidden=allow_hidden)
        if profile is None:
            return
        try:
            self.ps.getProfileDependencyChain(profile["id"])
        except KeyError as e:
            # Don't show twice the same error: old install and profile
            # oldinstall is test in first in other methods we may have an extra
            # 'Products.' in the namespace.
            #
            # TODO:
            # 1. Make sense of the previous comment.
            # 2. Possibly remove the special case for 'Products'.
            # 3. Make sense of the next five lines: they remove 'Products.'
            #    when it is there, and add it when it is not???
            checkname = product_id
            if checkname.startswith("Products."):
                checkname = checkname[9:]
            else:
                checkname = "Products." + checkname
            if checkname in self.errors:
                if self.errors[checkname]["value"] == e.args[0]:
                    return False
                # A new error is found, register it
                self.errors[product_id] = dict(
                    type=_("dependency_missing", default="Missing dependency"),
                    value=e.args[0],
                    product_id=product_id,
                )
            else:
                self.errors[product_id] = dict(
                    type=_("dependency_missing", default="Missing dependency"),
                    value=e.args[0],
                    product_id=product_id,
                )
            return False
        return True

    def get_product_version(self, product_id):
        """Return the version of the product (package).

        That implementation used to fall back to getting the version.txt.
        """
        try:
            dist = pkg_resources.get_distribution(product_id)
            return dist.version
        except pkg_resources.DistributionNotFound:
            if "." in product_id:
                return ""
        # For CMFPlacefulWorkflow we need to try Products.CMFPlacefulWorkflow.
        return self.get_product_version("Products." + product_id)

    def get_latest_upgrade_step(self, profile_id):
        """Get highest ordered upgrade step for profile.

        If anything errors out then go back to "old way" by returning
        'unknown'.
        """
        profile_version = UNKNOWN
        try:
            available = self.ps.listUpgrades(profile_id, True)
            if available:  # could return empty sequence
                latest = available[-1]
                profile_version = max(latest["dest"], key=pkg_resources.parse_version)
        except Exception:
            pass
        return profile_version

    def upgrade_info(self, product_id):
        """Returns upgrade info for a product.

        This is a dict with among others two booleans values, stating if
        an upgrade is required and available.

        :param product_id: id of product/package
        :type product_id: string
        :returns: dictionary with info about product
        :rtype: dict
        """
        available = self.is_product_installable(product_id, allow_hidden=True)
        if not available:
            return {}
        profile = self.get_install_profile(product_id, allow_hidden=True)
        if profile is None:
            # No GS profile, not supported.
            return {}
        profile_id = profile["id"]
        if not self.is_profile_installed(profile_id):
            return {}
        profile_version = str(self.ps.getVersionForProfile(profile_id))
        if profile_version == "latest":
            profile_version = self.get_latest_upgrade_step(profile_id)
        if profile_version == UNKNOWN:
            # If a profile doesn't have a metadata.xml use the package version.
            profile_version = self.get_product_version(product_id)
        installed_profile_version = self.ps.getLastVersionForProfile(profile_id)
        # getLastVersionForProfile returns the version as a tuple or unknown.
        if installed_profile_version != UNKNOWN:
            installed_profile_version = str(".".join(installed_profile_version))
        return dict(
            required=profile_version != installed_profile_version,
            available=len(self.ps.listUpgrades(profile_id)) > 0,
            hasProfile=True,  # TODO hasProfile is always True now.
            installedVersion=installed_profile_version,
            newVersion=profile_version,
        )

    def upgrade_product(self, product_id):
        """Run the upgrade steps for a product.

        Returns True on success, False otherwise.
        """
        profile = self.get_install_profile(product_id, allow_hidden=True)
        if profile is None:
            logger.error("Could not upgrade %s, no profile.", product_id)
            return False
        self.ps.upgradeProfile(profile["id"])
        return True

    def install_product(self, product_id, allow_hidden=False):
        """Install a product by name.

        :param product_id: id of product/package
        :type product_id: string
        :param allow_hidden: Allow installing otherwise hidden products.
            In the UI this will be False, but you can set it to True in
            for example a call from plone.app.upgrade where you want to
            install a new core product, even though it is hidden for users.
        :type allow_hidden: boolean
        :returns: True on success, False otherwise.
        :rtype: boolean
        """
        profile = self.get_install_profile(product_id, allow_hidden=allow_hidden)
        if not profile:
            logger.error("Could not install %s: no profile found.", product_id)
            # TODO Possibly raise an error.
            return False

        if self.is_product_installed(product_id):
            logger.error("Could not install %s: profile already installed.", product_id)
            return False

        # Okay, actually install the profile.
        profile_id = profile["id"]
        self.ps.runAllImportStepsFromProfile("profile-%s" % profile_id)

        if not self.is_profile_installed(profile_id):
            version = self.get_product_version(product_id)
            logger.warn(
                "Profile %s has no metadata.xml version. Falling back "
                "to package version %s",
                profile_id,
                version,
            )
            self.ps.setLastVersionForProfile(profile_id, version)

        # No problems encountered.
        return True

    def uninstall_product(self, product_id):
        """Uninstall a product by name.

        Returns True on success, False otherwise.
        """
        profile = self.get_uninstall_profile(product_id)
        if not profile:
            logger.error(
                "Could not uninstall %s: no uninstall profile " "found.", product_id
            )
            return False

        self.ps.runAllImportStepsFromProfile("profile-%s" % profile["id"])

        # Unmark the install profile.
        install_profile = self.get_install_profile(product_id, allow_hidden=True)
        if install_profile:
            self.ps.unsetLastVersionForProfile(install_profile["id"])
        return True

    @view.memoize
    def marshall_addons(self):
        addons = {}

        ignore_profiles = []
        ignore_products = []
        utils = getAllUtilitiesRegisteredFor(INonInstallable)
        for util in utils:
            ni_profiles = getattr(util, "getNonInstallableProfiles", None)
            if ni_profiles is not None:
                ignore_profiles.extend(ni_profiles())
            ni_products = getattr(util, "getNonInstallableProducts", None)
            if ni_products is not None:
                ignore_products.extend(ni_products())

        # Known profiles:
        profiles = self.ps.listProfileInfo()
        # Profiles that have upgrade steps (which may or may not have been
        # applied already).
        # profiles_with_upgrades = self.ps.listProfilesWithUpgrades()
        for profile in profiles:
            if profile["type"] != EXTENSION:
                continue

            pid = profile["id"]
            if pid in ignore_profiles:
                continue
            pid_parts = pid.split(":")
            if len(pid_parts) != 2:
                logger.error("Profile with id '%s' is invalid." % pid)
            # Which package (product) is this from?
            product_id = profile["product"]
            if product_id in ignore_products:
                continue
            profile_type = pid_parts[-1]
            if product_id not in addons:
                # get some basic information on the product
                installed = self.is_product_installed(product_id)
                upgrade_info = {}
                if installed:
                    upgrade_info = self.upgrade_info(product_id)
                elif not self.is_product_installable(product_id):
                    continue
                addons[product_id] = {
                    "id": product_id,
                    "version": self.get_product_version(product_id),
                    "title": product_id,
                    "description": "",
                    "upgrade_profiles": {},
                    "other_profiles": [],
                    "install_profile": None,
                    "install_profile_id": "",
                    "uninstall_profile": None,
                    "uninstall_profile_id": "",
                    "is_installed": installed,
                    "upgrade_info": upgrade_info,
                    "profile_type": profile_type,
                }
                # Add info on install and uninstall profile.
                product = addons[product_id]
                install_profile = self.get_install_profile(product_id)
                if install_profile is not None:
                    product["title"] = install_profile["title"]
                    product["description"] = install_profile["description"]
                    product["install_profile"] = install_profile
                    product["install_profile_id"] = install_profile["id"]
                    product["profile_type"] = "default"
                uninstall_profile = self.get_uninstall_profile(product_id)
                if uninstall_profile is not None:
                    product["uninstall_profile"] = uninstall_profile
                    product["uninstall_profile_id"] = uninstall_profile["id"]
                    # Do not override profile_type.
                    if not product["profile_type"]:
                        product["profile_type"] = "uninstall"
            if profile["id"] in (
                product["install_profile_id"],
                product["uninstall_profile_id"],
            ):
                # Everything has been done.
                continue
            elif "version" in profile:
                product["upgrade_profiles"][profile["version"]] = profile
            else:
                product["other_profiles"].append(profile)
        return addons

    def get_addons(self, apply_filter=None, product_name=None):
        """
        Based on generic setup profiles.

        @filter:= 'installed': only products that are installed and not hidden
                  'upgrades': only products with upgrades
                  'available': products that are not installed bit
                               could be
                  'broken': uninstallable products with broken
                            dependencies

        @product_name:= a specific product id that you want info on. Do
                   not pass in the profile type, just the name

        XXX: I am pretty sure we don't want base profiles ...
        """
        addons = self.marshall_addons()
        filtered = {}
        if apply_filter == "broken":
            all_broken = self.errors.values()
            for broken in all_broken:
                filtered[broken["product_id"]] = broken
        else:
            for product_id, addon in addons.items():
                if product_name and addon["id"] != product_name:
                    continue

                installed = addon["is_installed"]
                if apply_filter in ["installed", "upgrades"] and not installed:
                    continue
                elif apply_filter == "available":
                    if installed:
                        continue
                    # filter out upgrade profiles
                    if addon["profile_type"] != "default":
                        continue
                elif apply_filter == "upgrades":
                    upgrade_info = addon["upgrade_info"]
                    if not upgrade_info.get("available"):
                        continue

                filtered[product_id] = addon

        return filtered



from plone.restapi.deserializer import json_body
from plone.restapi.interfaces import IPloneRestapiLayer
from plone.restapi.services import Service
from plone.restapi.types.utils import serializeSchema
from plone.restapi.types.utils import add_field
from plone.restapi.types.utils import add_fieldset
from plone.restapi.types.utils import get_info_for_type
from plone.restapi.types.utils import update_field
from plone.restapi.types.utils import update_fieldset
from zExceptions import BadRequest
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.interface import noLongerProvides
from zope.publisher.interfaces import IPublishTraverse
import plone.protect.interfaces


@implementer(IPublishTraverse)
class TypesUpdate(Service):
    def __init__(self, context, request):
        super().__init__(context, request)
        self.params = []

    def publishTraverse(self, request, name):
        # Treat any path segments after /@types as parameters
        self.params.append(name)
        return self

    def reply(self):
        if not self.params:
            raise BadRequest("Missing parameter typename")

        # Disable CSRF protection
        if "IDisableCSRFProtection" in dir(plone.protect.interfaces):
            alsoProvides(self.request, plone.protect.interfaces.IDisableCSRFProtection)

        # Make sure we get the right dexterity-types adapter
        if IPloneRestapiLayer.providedBy(self.request):
            noLongerProvides(self.request, IPloneRestapiLayer)

        data = json_body(self.request)
        if len(self.params) == 1:
            name = self.params.pop(0)
        elif len(self.params) == 2:
            name = self.params.pop(0)
            fname = self.params.pop(0)
            if "fields" in data:
                return self.reply_for_fieldset(name, fname, data)
            return self.reply_for_field(name, fname, data)
        else:
            raise BadRequest("Too many parameters")

        context = queryMultiAdapter(
            (self.context, self.request), name="dexterity-types"
        )

        # Get content type SchemaContext
        context = context.publishTraverse(self.request, name)

        # Update Field properties
        properties = data.get("properties", {})
        for key, value in properties.items():
            self.reply_for_field(name, key, value, create=True)

        # Update Fieldset properties
        fieldsets = data.get("fieldsets", [])
        for fieldset in fieldsets:
            fname = fieldset.get("id")
            self.reply_for_fieldset(name, fname, fieldset, create=True)

        return self.reply_no_content()

    def reply_for_fieldset(self, name, fieldset_name, data, create=False):
        context = queryMultiAdapter(
            (self.context, self.request), name="dexterity-types"
        )

        # Get content type SchemaContext
        context = context.publishTraverse(self.request, name)

        data["id"] = fieldset_name

        if create:
            info = get_info_for_type(context, self.request, name)
            existing = {f.get("id") for f in info.get("fieldsets", [])}
            if fieldset_name not in existing:
                add_fieldset(context, self.request, data)
        update_fieldset(context, self.request, data)

        serializeSchema(context.schema)
        return self.reply_no_content()

    def reply_for_field(self, name, field_name, data, create=False):
        context = queryMultiAdapter(
            (self.context, self.request), name="dexterity-types"
        )

        # Get content type SchemaContext
        context = context.publishTraverse(self.request, name)

        data["id"] = field_name

        if create:
            info = get_info_for_type(context, self.request, name)
            existing = info.get("properties", {})
            if field_name not in existing:
                add_field(context, self.request, data)

        if field_name in context.schema:
            update_field(context, self.request, data)

        serializeSchema(context.schema)
        return self.reply_no_content()



from plone.restapi.interfaces import IExpandableElement
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.services import Service
from Products.CMFCore.interfaces._content import IWorkflowAware
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(IExpandableElement)
@adapter(IWorkflowAware, Interface)
class WorkflowInfo:
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, expand=False):
        result = {"workflow": {"@id": f"{self.context.absolute_url()}/@workflow"}}
        if not expand:
            return result

        # Prevent 404 on site root on workflow request
        # Although 404 will be more semantic, for the sake of uniformity of the
        # API we fake the response to the endpoint by providing an empty
        # response instead of a 404.
        if IPloneSiteRoot.providedBy(self.context):
            result["workflow"].update({"history": [], "transitions": []})
            return result

        wftool = getToolByName(self.context, "portal_workflow")
        try:
            history = wftool.getInfoFor(self.context, "review_history")
        except WorkflowException:
            history = []

        actions = wftool.listActionInfos(object=self.context)
        transitions = []
        for action in actions:
            if action["category"] != "workflow":
                continue

            title = action["title"]
            if isinstance(title, bytes):
                title = title.decode("utf8")

            transitions.append(
                {
                    "@id": "{}/@workflow/{}".format(
                        self.context.absolute_url(), action["id"]
                    ),
                    "title": self.context.translate(title),
                }
            )

        for item, action in enumerate(history):
            title = wftool.getTitleForStateOnType(
                action["review_state"], self.context.portal_type
            )
            if isinstance(title, bytes):
                title = title.decode("utf8")
            history[item]["title"] = self.context.translate(title)

        try:
            current_state = wftool.getInfoFor(self.context, "review_state")
        except WorkflowException:
            current_state = ""
        current_state_title = wftool.getTitleForStateOnType(
            current_state,
            self.context.portal_type,
        )

        result["workflow"].update(
            {
                "history": json_compatible(history),
                "transitions": transitions,
                "state": {
                    "id": current_state,
                    "title": self.context.translate(current_state_title),
                },
            }
        )
        return result


class WorkflowInfoService(Service):
    """Get workflow information"""

    def reply(self):
        info = WorkflowInfo(self.context, self.request)
        return info(expand=True)["workflow"]



from plone.restapi.deserializer import boolean_value
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

import unittest


class TestBooleanValue(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def test_true_bool(self):
        self.assertTrue(boolean_value(True))

    def test_true_string(self):
        self.assertTrue(boolean_value("true"))

    def test_true_string_uppercase(self):
        self.assertTrue(boolean_value("True"))

    def test_true_int(self):
        self.assertTrue(boolean_value(1))

    def test_true_int_string(self):
        self.assertTrue(boolean_value("1"))

    def test_false_bool(self):
        self.assertFalse(boolean_value(False))

    def test_false_string(self):
        self.assertFalse(boolean_value("false"))

    def test_false_string_uppercase(self):
        self.assertFalse(boolean_value("False"))

    def test_false_int(self):
        self.assertFalse(boolean_value(0))

    def test_false_int_string(self):
        self.assertFalse(boolean_value("0"))

    def test_true_other_value(self):
        self.assertTrue(boolean_value("foobar"))



from plone import api
from plone.app.testing import TEST_USER_ID
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.testing import PLONE_RESTAPI_DX_INTEGRATION_TESTING
from zope.component import getMultiAdapter

import unittest


class TestSerializeUserToJsonAdapters(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.gtool = api.portal.get_tool("portal_groups")
        properties = {
            "title": "Plone Team",
            "description": "We are Plone",
            "email": "ploneteam@plone.org",
        }
        self.gtool.addGroup(
            "ploneteam",
            (),
            (),
            properties=properties,
            title=properties["title"],
            description=properties["description"],
        )
        self.group = self.gtool.getGroupById("ploneteam")
        self.group.addMember(TEST_USER_ID)

    def serialize(self, user):
        serializer = getMultiAdapter((user, self.request), ISerializeToJson)
        return serializer()

    def serialize_summary(self, user):
        serializer = getMultiAdapter((user, self.request), ISerializeToJsonSummary)
        return serializer()

    def test_serialize_returns_id(self):
        group = self.serialize(self.group)
        self.assertTrue(group)
        self.assertEqual("ploneteam", group.get("id"))
        self.assertEqual("ploneteam@plone.org", group.get("email"))
        self.assertEqual("Plone Team", group.get("title"))
        self.assertEqual("We are Plone", group.get("description"))
        self.assertEqual(set(group["users"]), {"@id", "items_total", "items"})

    def test_summary(self):
        group = self.serialize_summary(self.group)
        self.assertTrue(group)
        self.assertEqual("ploneteam", group.get("id"))
        self.assertEqual("ploneteam@plone.org", group.get("email"))
        self.assertEqual("Plone Team", group.get("title"))
        self.assertEqual("We are Plone", group.get("description"))
        self.assertNotIn("users", group)



from Products.Five.browser import BrowserView  # pragma: no cover


class InternalServerErrorView(BrowserView):  # pragma: no cover
    def __call__(self):
        from urllib.error import HTTPError

        raise HTTPError(
            "http://nohost/plone/internal_server_error",
            500,
            "InternalServerError",
            {},
            None,
        )



from OFS.Image import Image
from plone import api
from plone.app.discussion.interfaces import IConversation
from plone.app.discussion.interfaces import IDiscussionSettings
from plone.app.discussion.interfaces import IReplies
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.testing import PLONE_RESTAPI_DX_INTEGRATION_TESTING
from Products.CMFCore.utils import getToolByName
from Products.PlonePAS.tests import dummy
from unittest import TestCase
from zope.component import createObject
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryUtility


class TestCommentsSerializers(TestCase):
    layer = PLONE_RESTAPI_DX_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()

        # Allow discussion
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IDiscussionSettings, check=False)
        settings.globally_enabled = True
        settings.edit_comment_enabled = True
        settings.delete_own_comment_enabled = True

        # doc with comments
        self.doc = api.content.create(
            container=self.portal,
            type="Document",
            id="doc_with_comments",
            title="Document with comments",
            allow_discussion=True,
        )
        self.conversation = IConversation(self.doc)
        self.replies = IReplies(self.conversation)
        comment = createObject("plone.Comment")
        comment.text = "Comment"
        self.comment = self.replies[self.replies.addComment(comment)]

        comment = createObject("plone.Comment")
        comment.text = "Comment 2"
        self.replies.addComment(comment)

    def test_conversation(self):
        serializer = getMultiAdapter(
            (self.conversation, self.request), ISerializeToJson
        )

        output = serializer()
        self.assertEqual(set(output), {"@id", "items_total", "items"})

    def test_conversation_batched(self):
        self.request.form["b_size"] = 1
        serializer = getMultiAdapter(
            (self.conversation, self.request), ISerializeToJson
        )

        output = serializer()
        self.assertIn("batching", output)

    def test_comment(self):
        serializer = getMultiAdapter((self.comment, self.request), ISerializeToJson)

        output = serializer()

        expected = [
            "@id",
            "@type",
            "@parent",
            "comment_id",
            "in_reply_to",
            "text",
            "user_notification",
            "author_username",
            "author_name",
            "author_image",
            "creation_date",
            "modification_date",
            "is_editable",
            "is_deletable",
        ]
        self.assertEqual(set(output), set(expected))

        self.assertEqual(set(output["text"]), {"data", "mime-type"})

    def test_comment_with_author_image(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        # set member portrait
        membertool = getToolByName(self.portal, "portal_memberdata")
        membertool._setPortrait(
            Image(id=TEST_USER_ID, file=dummy.File(), title=""), TEST_USER_ID
        )
        self.conversation = IConversation(self.doc)
        self.replies = IReplies(self.conversation)
        comment = createObject("plone.Comment")
        comment.text = "Hey ho, let's go!"
        comment.author_username = TEST_USER_ID
        self.comment = self.replies[self.replies.addComment(comment)]

        serializer = getMultiAdapter((self.comment, self.request), ISerializeToJson)
        self.assertEqual(
            f"{self.portal_url}/portal_memberdata/portraits/test_user_1_",
            serializer().get("author_image"),
        )

    def test_comment_with_no_author_image(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.conversation = IConversation(self.doc)
        self.replies = IReplies(self.conversation)
        comment = createObject("plone.Comment")
        comment.text = "Hey ho, let's go!"
        comment.author_username = TEST_USER_ID
        self.comment = self.replies[self.replies.addComment(comment)]

        serializer = getMultiAdapter((self.comment, self.request), ISerializeToJson)
        self.assertEqual(
            None,
            serializer().get("author_image"),
        )

    def test_comment_with_mimetype_text_plain(self):
        self.conversation = IConversation(self.doc)
        self.replies = IReplies(self.conversation)
        comment = createObject("plone.Comment")
        comment.text = "Hey, I am plain text!"
        comment.mime_type = "text/plain"
        self.comment = self.replies[self.replies.addComment(comment)]

        serializer = getMultiAdapter((self.comment, self.request), ISerializeToJson)

        # serializer should return HTML with a clickable link
        self.assertEqual(
            "Hey, I am plain text!",
            serializer()["text"]["data"],
        )
        # serializer should return mimetype = text/x-web-intelligent
        self.assertEqual("text/plain", serializer()["text"]["mime-type"])

    def test_comment_with_mimetype_intelligent_text(self):
        # Set text transform to intelligent text
        registry = queryUtility(IRegistry)
        settings = registry.forInterface(IDiscussionSettings, check=False)
        settings.text_transform = "text/x-web-intelligent"

        self.conversation = IConversation(self.doc)
        self.replies = IReplies(self.conversation)
        comment = createObject("plone.Comment")
        comment.text = "Go to https://www.plone.org"
        comment.mime_type = "text/x-web-intelligent"
        self.comment = self.replies[self.replies.addComment(comment)]

        serializer = getMultiAdapter((self.comment, self.request), ISerializeToJson)

        # serializer should return HTML with a clickable link
        self.assertEqual(
            'Go to <a href="https://www.plone.org" '
            + 'rel="nofollow">https://www.plone.org</a>',
            serializer()["text"]["data"],
        )
        # serializer should return mimetype = text/html
        self.assertEqual("text/html", serializer()["text"]["mime-type"])

    def test_comment_with_mimetype_html(self):
        # Set text transform to text/html
        registry = queryUtility(IRegistry)
        settings = registry.forInterface(IDiscussionSettings, check=False)
        settings.text_transform = "text/html"

        self.conversation = IConversation(self.doc)
        self.replies = IReplies(self.conversation)
        comment = createObject("plone.Comment")
        comment.text = "Go to <a href='https://www.plone.org'>Plone</a>"
        comment.mime_type = "text/html"
        self.comment = self.replies[self.replies.addComment(comment)]

        serializer = getMultiAdapter((self.comment, self.request), ISerializeToJson)

        # serializer should return HTML
        self.assertEqual(
            'Go to <a href="https://www.plone.org">Plone</a>',
            serializer()["text"]["data"],
        )
        # serializer should return mimetype = text/html
        self.assertEqual("text/html", serializer()["text"]["mime-type"])



from unittest.mock import patch
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.namedfile.file import NamedBlobFile
from plone.namedfile.file import NamedBlobImage
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING
from plone.restapi.testing import RelativeSession
from plone.scale import storage
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import os
import transaction
import unittest


class TestTraversal(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url, test=self)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_get_document(self):
        self.portal.invokeFactory("Document", id="doc1", title="My Document")
        self.portal.doc1.description = "This is a document"
        self.portal.doc1.text = RichTextValue("Lorem ipsum", "text/plain", "text/html")
        transaction.commit()

        response = self.api_session.get(self.portal.doc1.absolute_url())

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.headers.get("Content-Type"),
            "application/json",
            "When sending a GET request with Accept: application/json "
            + "the server should respond with sending back application/json: "
            + "{}".format(response.headers.get("Content-Type")),
        )
        self.assertEqual(
            "Document",
            response.json().get("@type"),
            "Response should be @type 'Document', not '{}'".format(
                response.json().get("@type")
            ),
        )
        self.assertEqual(
            response.json().get("@id"),
            self.portal.doc1.absolute_url(),
            "@id attribute != {}: {}".format(
                self.portal.doc1.absolute_url(), response.json()
            ),
        )
        self.assertEqual("My Document", response.json().get("title"))
        self.assertEqual("This is a document", response.json().get("description"))
        self.assertEqual(
            {
                "data": "<p>Lorem ipsum</p>",
                "content-type": "text/plain",
                "encoding": "utf-8",
            },
            response.json().get("text"),
        )

    def test_get_news_item(self):
        self.portal.invokeFactory("News Item", id="news1", title="News Item 1")
        image_file = os.path.join(os.path.dirname(__file__), "image.png")
        with open(image_file, "rb") as f:
            image_data = f.read()
        self.portal.news1.image = NamedBlobImage(
            data=image_data, contentType="image/png", filename="image.png"
        )
        self.portal.news1.image_caption = "This is an image caption."
        transaction.commit()

        with patch.object(storage, "uuid4", return_value="uuid1"):
            response = self.api_session.get(self.portal.news1.absolute_url())

            self.assertEqual(response.status_code, 200)
            self.assertEqual(
                response.headers.get("Content-Type"),
                "application/json",
                "When sending a GET request with Content-Type: application/json "
                + "the server should respond with sending back application/json.",  # noqa
            )
            self.assertEqual(
                "News Item",
                response.json().get("@type"),
                "Response should be @type 'News Item', not '{}'".format(
                    response.json().get("@type")
                ),
            )
            self.assertEqual(
                response.json().get("@id"), self.portal.news1.absolute_url()
            )
            self.assertEqual("News Item 1", response.json().get("title"))
            self.assertEqual(
                "This is an image caption.", response.json()["image_caption"]
            )
            self.assertDictContainsSubset(
                {"download": self.portal_url + "/news1/@@images/uuid1.png"},  # noqa
                response.json()["image"],
            )

    def test_get_folder(self):
        self.portal.invokeFactory("Folder", id="folder1", title="My Folder")
        transaction.commit()

        response = self.api_session.get(self.portal.folder1.absolute_url())

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.headers.get("Content-Type"),
            "application/json",
            "When sending a GET request with Content-Type: application/json "
            + "the server should respond with sending back application/json.",
        )
        self.assertEqual(
            "Folder",
            response.json().get("@type"),
            "Response should be @type 'Folder', not '{}'".format(
                response.json().get("@type")
            ),
        )
        self.assertEqual(self.portal.folder1.absolute_url(), response.json().get("@id"))
        self.assertEqual("My Folder", response.json().get("title"))

    def test_get_site_root(self):
        response = self.api_session.get(self.portal_url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.headers.get("Content-Type"),
            "application/json",
            "When sending a GET request with Content-Type: application/json "
            + "the server should respond with sending back application/json.",
        )
        self.assertEqual(self.portal_url, response.json().get("@id"))
        self.assertEqual("Plone Site", response.json().get("@type"))

    def test_get_site_root_with_default_page(self):
        self.portal.invokeFactory("Document", id="front-page")
        self.portal.setDefaultPage("front-page")
        transaction.commit()

        response = self.api_session.get(self.portal_url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.headers.get("Content-Type"),
            "application/json",
            "When sending a GET request with Content-Type: application/json "
            + "the server should respond with sending back application/json.",
        )
        self.assertEqual(response.json().get("@id"), self.portal_url)
        self.assertEqual("Plone Site", response.json().get("@type"))

    @unittest.skip("Not implemented yet.")
    def test_get_file(self):  # pragma: no cover
        self.portal.invokeFactory("File", id="file1")
        self.portal.file1.title = "File"
        self.portal.file1.description = "A file"
        pdf_file = os.path.join(os.path.dirname(__file__), "file.pdf")
        with open(pdf_file, "rb") as f:
            pdf_data = f.read()
        self.portal.file1.file = NamedBlobFile(
            data=pdf_data, contentType="application/pdf", filename="file.pdf"
        )
        intids = getUtility(IIntIds)
        file_id = intids.getId(self.portal.file1)
        self.portal.file1.file = RelationValue(file_id)
        transaction.commit()

        response = self.api_session.get(self.portal.file1.absolute_url())

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.headers.get("Content-Type"),
            "application/json",
            "When sending a GET request with Content-Type: application/json "
            + "the server should respond with sending back application/json.",
        )
        self.assertEqual(response.json()["@id"], self.portal.file1.absolute_url())

    @unittest.skip("Not implemented yet.")
    def test_get_image(self):  # pragma: no cover
        self.portal.invokeFactory("Image", id="img1")
        self.portal.img1.title = "Image"
        self.portal.img1.description = "An image"
        image_file = os.path.join(os.path.dirname(__file__), "image.png")
        with open(image_file, "rb") as f:
            image_data = f.read()
        self.portal.img1.image = NamedBlobImage(
            data=image_data, contentType="image/png", filename="image.png"
        )
        transaction.commit()

        response = self.api_session.get(self.portal.img1.absolute_url())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.headers.get("Content-Type"),
            "application/json",
            "When sending a GET request with Content-Type: application/json "
            + "the server should respond with sending back application/json.",
        )
        self.assertEqual(response.json()["@id"], self.portal.img1.absolute_url())



from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.types import utils
from plone.tiles.interfaces import ITileType
from zope.component import adapter
from zope.component.hooks import getSite
from zope.interface import implementer
from zope.interface import Interface


SERVICE_ID = "@tiles"


@implementer(ISerializeToJsonSummary)
@adapter(ITileType, Interface)
class TileSummarySerializeToJson:
    def __init__(self, tile, request):
        self.tile = tile

    def __call__(self):
        portal = getSite()
        return {
            "@id": "{}/{}/{}".format(
                portal.absolute_url(), SERVICE_ID, self.tile.__name__
            ),
            "title": self.tile.title,
            "description": self.tile.description,
            "icon": self.tile.icon,
        }


def get_jsonschema_for_tile(tile, context, request):
    """Build a complete JSON schema for the given tile."""
    schema = tile.schema

    fieldsets = utils.get_fieldsets(context, request, schema)

    # Build JSON schema properties
    properties = utils.get_jsonschema_properties(context, request, fieldsets)

    # Determine required fields
    required = []
    for field in utils.iter_fields(fieldsets):
        if field.field.required:
            required.append(field.field.getName())

    # Include field modes
    for field in utils.iter_fields(fieldsets):
        if field.mode:
            properties[field.field.getName()]["mode"] = field.mode

    return {
        "type": "object",
        "title": tile.title,
        "properties": properties,
        "required": required,
        "fieldsets": utils.get_fieldset_infos(fieldsets),
    }


@implementer(ISerializeToJson)
@adapter(ITileType, Interface)
class TileSerializeToJson:
    def __init__(self, tile, request):
        self.tile = tile
        self.request = request
        self.schema = self.tile.schema

    def __call__(self):
        portal = getSite()

        return get_jsonschema_for_tile(self.tile, portal, self.request)



from plone.registry.interfaces import IRegistry
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import IZCatalogCompatibleQuery
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.browser.navtree import getNavigationRoot
from Products.CMFPlone.interfaces import ISearchSchema
from zope.component import getMultiAdapter
from zope.component import getUtility


class SearchHandler:
    """Executes a catalog search based on a query dict, and returns
    JSON compatible results.
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.catalog = getToolByName(self.context, "portal_catalog")

    def _parse_query(self, query):
        catalog_compatible_query = getMultiAdapter(
            (self.context, self.request), IZCatalogCompatibleQuery
        )(query)
        return catalog_compatible_query

    def _constrain_query_by_path(self, query):
        """If no 'path' query was supplied, restrict search to current
        context and its children by adding a path constraint.

        The following cases can happen here:
        - No 'path' parameter at all
        - 'path' query dict with options, but no actual 'query' inside
        - 'path' supplied as a string
        - 'path' supplied as a complete query dict
        """
        if "path" not in query:
            query["path"] = {}

        if isinstance(query["path"], str) or isinstance(query["path"], list):
            query["path"] = {"query": query["path"]}

        # If this is accessed through a VHM the client does not know
        # the complete physical path of an object. But the path index
        # indexes the complete physical path. Complete the path.
        vhm_physical_path = self.request.get("VirtualRootPhysicalPath")
        if vhm_physical_path:
            path = query["path"].get("query")
            if path:
                if isinstance(path, str):
                    path = path.lstrip("/")
                    full_path = "/".join(vhm_physical_path + (path,))
                    query["path"]["query"] = full_path
                if isinstance(path, list):
                    full_paths = []
                    for p in path:
                        p = p.lstrip("/")
                        full_path = "/".join(vhm_physical_path + (p,))
                        full_paths.append(full_path)
                    query["path"]["query"] = full_paths

        if isinstance(query["path"], dict) and "query" not in query["path"]:
            # We either had no 'path' parameter at all, or an incomplete
            # 'path' query dict (with just ExtendedPathIndex options (like
            # 'depth'), but no actual path 'query' in it).
            #
            # In either case, we'll prefill with the context's path
            path = "/".join(self.context.getPhysicalPath())
            query["path"]["query"] = path

    def search(self, query=None):
        if query is None:
            query = {}
        if "fullobjects" in query:
            fullobjects = True
            del query["fullobjects"]
        else:
            fullobjects = False

        use_site_search_settings = False

        if "use_site_search_settings" in query:
            use_site_search_settings = True
            del query["use_site_search_settings"]

        if use_site_search_settings:
            query = self.filter_query(query)

        self._constrain_query_by_path(query)
        query = self._parse_query(query)

        lazy_resultset = self.catalog.searchResults(**query)
        results = getMultiAdapter((lazy_resultset, self.request), ISerializeToJson)(
            fullobjects=fullobjects
        )

        return results

    def filter_types(self, types):
        plone_utils = getToolByName(self.context, "plone_utils")
        if not isinstance(types, list):
            types = [types]
        return plone_utils.getUserFriendlyTypes(types)

    def filter_query(self, query):
        registry = getUtility(IRegistry)
        search_settings = registry.forInterface(ISearchSchema, prefix="plone")

        types = query.get("portal_type", [])
        if "query" in types:
            types = types["query"]
        query["portal_type"] = self.filter_types(types)

        # respect effective/expiration date
        query["show_inactive"] = False

        # respect navigation root
        if "path" not in query:
            query["path"] = {"query": getNavigationRoot(self.context)}

            vhm_physical_path = self.request.get("VirtualRootPhysicalPath")
            # if vhm trick is applied, we should present a stripped path, as it will be
            # processed again in _constrain_query_by_path
            if vhm_physical_path:
                bits = query["path"]["query"].split("/")[len(vhm_physical_path) :]
                query["path"]["query"] = "/".join(bits) or "/"

        default_sort_on = search_settings.sort_on

        if "sort_on" not in query:
            if default_sort_on != "relevance":
                query["sort_on"] = self.default_sort_on
        elif query["sort_on"] == "relevance":
            del query["sort_on"]

        if not query.get("sort_order") and (
            query.get("sort_on", "") == "Date"
            or query.get("sort_on", "") == "effective"  # compatibility with Volto
        ):
            query["sort_order"] = "reverse"
        elif "sort_order" in query:
            del query["sort_order"]

        if "sort_order" in query and not query["sort_order"]:
            del query["sort_order"]

        return query



from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.namedfile.file import NamedBlobImage
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING
from Products.CMFCore.utils import getToolByName
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import os
import requests
import transaction
import unittest


class TestContentGet(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Member"])
        login(self.portal, SITE_OWNER_NAME)
        self.portal.invokeFactory("Folder", id="folder1", title="My Folder")
        self.portal.folder1.invokeFactory("Document", id="doc1", title="My Document")
        self.portal.folder1.doc1.text = RichTextValue(
            "Lorem ipsum.", "text/plain", "text/html"
        )
        self.portal.folder1.invokeFactory("Folder", id="folder2", title="My Folder 2")
        self.portal.folder1.folder2.invokeFactory(
            "Document", id="doc2", title="My Document 2"
        )
        self.portal.folder1.invokeFactory(
            "Collection", id="collection", title="My collection"
        )
        wftool = getToolByName(self.portal, "portal_workflow")
        wftool.doActionFor(self.portal.folder1, "publish")
        wftool.doActionFor(self.portal.folder1.doc1, "publish")
        wftool.doActionFor(self.portal.folder1.folder2, "publish")
        wftool.doActionFor(self.portal.folder1.folder2.doc2, "publish")
        transaction.commit()

    def test_get_content_returns_fullobjects(self):
        response = requests.get(
            self.portal.folder1.absolute_url() + "?fullobjects",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(3, len(response.json()["items"]))
        self.assertTrue("title" in list(response.json()["items"][0]))
        self.assertTrue("description" in list(response.json()["items"][0]))
        self.assertTrue("text" in list(response.json()["items"][0]))
        self.assertEqual(
            {
                "data": "<p>Lorem ipsum.</p>",
                "content-type": "text/plain",
                "encoding": "utf-8",
            },
            response.json()["items"][0].get("text"),
        )

        # make sure the single document response is the same as the items
        response_doc = requests.get(
            self.portal.folder1.doc1.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(response.json()["items"][0], response_doc.json())

    def test_get_content_include_items(self):
        response = requests.get(
            self.portal.folder1.absolute_url() + "?include_items=false",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(response.status_code, 200)
        self.assertNotIn("items", response.json())

    def test_get_content_returns_fullobjects_correct_id(self):
        response = requests.get(
            self.portal.folder1.absolute_url() + "?fullobjects",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(3, len(response.json()["items"]))
        self.assertEqual(
            response.json()["items"][1]["@id"], self.portal_url + "/folder1/folder2"
        )

    def test_get_content_returns_fullobjects_non_recursive(self):
        response = requests.get(
            self.portal.folder1.absolute_url() + "?fullobjects",
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(3, len(response.json()["items"]))
        self.assertTrue("items" not in response.json()["items"][1])

    def test_get_content_includes_related_items(self):
        intids = getUtility(IIntIds)
        self.portal.folder1.doc1.relatedItems = [
            RelationValue(intids.getId(self.portal.folder1.folder2.doc2))
        ]
        transaction.commit()
        response = requests.get(
            self.portal.folder1.doc1.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(1, len(response.json()["relatedItems"]))
        self.assertEqual(
            [
                {
                    "@id": self.portal_url + "/folder1/folder2/doc2",
                    "@type": "Document",
                    "description": "",
                    "review_state": "published",
                    "title": "My Document 2",
                }
            ],
            response.json()["relatedItems"],
        )

    def test_get_content_related_items_without_workflow(self):
        intids = getUtility(IIntIds)

        self.portal.invokeFactory("Image", id="imagewf")
        self.portal.imagewf.title = "Image without workflow"
        self.portal.imagewf.description = "This is an image"
        image_file = os.path.join(os.path.dirname(__file__), "image.png")
        with open(image_file, "rb") as f:
            image_data = f.read()
        self.portal.imagewf.image = NamedBlobImage(
            data=image_data, contentType="image/png", filename="image.png"
        )
        transaction.commit()

        self.portal.folder1.doc1.relatedItems = [
            RelationValue(intids.getId(self.portal.imagewf))
        ]
        transaction.commit()
        response = requests.get(
            self.portal.folder1.doc1.absolute_url(),
            headers={"Accept": "application/json"},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(1, len(response.json()["relatedItems"]))
        self.assertEqual(
            [
                {
                    "@id": self.portal_url + "/imagewf",
                    "@type": "Image",
                    "description": "This is an image",
                    "review_state": None,
                    "title": "Image without workflow",
                }
            ],
            response.json()["relatedItems"],
        )



from datetime import date
from datetime import datetime
from datetime import time
from datetime import timedelta
from decimal import Decimal
from plone import namedfile
from plone.app.textfield.value import RichTextValue
from plone.dexterity.utils import iterSchemata
from plone.restapi.interfaces import IFieldDeserializer
from plone.restapi.testing import PLONE_RESTAPI_DX_INTEGRATION_TESTING
from plone.restapi.tests.dxtypes import IDXTestDocumentSchema
from plone.uuid.interfaces import IUUID
from pytz import timezone
from zope.component import getMultiAdapter
from zope.schema import Field
from zope.schema._bootstrapinterfaces import RequiredMissing
from zope.schema.interfaces import ConstraintNotSatisfied
from zope.schema.interfaces import ValidationError

import unittest


class RequiredField:
    """Context manager that will make a field required and back to old state."""

    def __init__(self, field):
        self.field = field
        self.old_state = field.required

    def __enter__(self):
        self.field.required = True

    def __exit__(self, *args, **kwargs):
        self.field.required = self.old_state


class TestDXFieldDeserializer(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        self.portal.invokeFactory("DXTestDocument", id="doc1", title="Test Document")

    def deserialize(self, fieldname, value):
        for schema in iterSchemata(self.portal.doc1):
            if fieldname in schema:
                field = schema.get(fieldname)
                break
        deserializer = getMultiAdapter(
            (field, self.portal.doc1, self.request), IFieldDeserializer
        )
        return deserializer(value)

    def test_ascii_deserialization_returns_native_string(self):
        value = self.deserialize("test_ascii_field", "Foo")
        self.assertTrue(isinstance(value, str), "Not a <str>")
        self.assertEqual("Foo", value)

    def test_asciiline_deserialization_returns_native_string(self):
        value = self.deserialize("test_asciiline_field", "Foo")
        self.assertTrue(isinstance(value, str), "Not a <str>")
        self.assertEqual("Foo", value)

    def test_bool_deserialization_returns_true(self):
        value = self.deserialize("test_bool_field", True)
        self.assertTrue(isinstance(value, bool), "Not a <bool>")
        self.assertEqual(True, value)

    def test_bool_deserialization_returns_false(self):
        value = self.deserialize("test_bool_field", False)
        self.assertTrue(isinstance(value, bool), "Not a <bool>")
        self.assertEqual(False, value)

    def test_bytes_deserialization_returns_bytestring(self):
        value = self.deserialize("test_bytes_field", "Foo")
        self.assertTrue(isinstance(value, bytes), "Not a <bytes>")
        self.assertEqual(b"Foo", value)

    def test_bytesline_deserialization_returns_bytestring(self):
        value = self.deserialize("test_bytesline_field", "Foo")
        self.assertTrue(isinstance(value, bytes), "Not a <bytes>")
        self.assertEqual(b"Foo", value)

    def test_choice_deserialization_returns_vocabulary_item(self):
        value = self.deserialize("test_choice_field", "bar")
        self.assertTrue(isinstance(value, str), "Not an <unicode>")
        self.assertEqual("bar", value)

    def test_choice_deserialization_from_token_returns_vocabulary_value(self):
        value = self.deserialize("test_choice_field_with_vocabulary", "token1")
        self.assertTrue(isinstance(value, str), "Not an <unicode>")
        self.assertEqual("value1", value)

    def test_choice_deserialization_from_value_returns_vocabulary_value(self):
        value = self.deserialize("test_choice_field_with_vocabulary", "value1")
        self.assertTrue(isinstance(value, str), "Not an <unicode>")
        self.assertEqual("value1", value)

    def test_choice_deserialization_from_term_returns_vocabulary_value(self):
        value = self.deserialize(
            "test_choice_field_with_vocabulary",
            {"token": "token1", "title": "title1"},
        )
        self.assertTrue(isinstance(value, str), "Not an <unicode>")
        self.assertEqual("value1", value)

    def test_date_deserialization_returns_date(self):
        value = self.deserialize("test_date_field", "2015-12-20")
        self.assertTrue(isinstance(value, date))
        self.assertEqual(date(2015, 12, 20), value)

    def test_datetime_deserialization_returns_datetime(self):
        value = self.deserialize("test_datetime_field", "2015-12-20T10:39:54.361Z")
        self.assertTrue(isinstance(value, datetime), "Not a <datetime>")
        self.assertEqual(datetime(2015, 12, 20, 10, 39, 54, 361000), value)

    def test_datetime_deserialization_handles_timezone(self):
        value = self.deserialize("test_datetime_field", "2015-12-20T10:39:54.361+01")
        self.assertEqual(datetime(2015, 12, 20, 9, 39, 54, 361000), value)

    def test_datetime_with_tz_deserialization_keeps_timezone(self):
        value = self.deserialize("test_datetime_tz_field", "2015-12-20T10:39:54.361+01")
        self.assertEqual(
            timezone("Europe/Zurich").localize(
                datetime(2015, 12, 20, 10, 39, 54, 361000)
            ),
            value,
        )

    def test_datetime_with_tz_deserialization_converts_timezone(self):
        value = self.deserialize("test_datetime_tz_field", "2015-12-20T10:39:54.361-04")
        self.assertEqual(
            timezone("Europe/Zurich").localize(
                datetime(2015, 12, 20, 15, 39, 54, 361000)
            ),
            value,
        )

    def test_datetime_with_tz_deserialization_adds_timezone(self):
        value = self.deserialize("test_datetime_tz_field", "2015-12-20T10:39:54.361")
        self.assertEqual(
            timezone("Europe/Zurich").localize(
                datetime(2015, 12, 20, 11, 39, 54, 361000)
            ),
            value,
        )

    def test_datetime_with_tz_deserialization_handles_dst(self):
        value = self.deserialize("test_datetime_tz_field", "2015-05-20T10:39:54.361+02")
        self.assertEqual(
            timezone("Europe/Zurich").localize(
                datetime(2015, 5, 20, 10, 39, 54, 361000)
            ),
            value,
        )

    def test_datetime_deserialization_none(self):
        # Make sure we don't construct a datetime out of nothing
        value = self.deserialize("test_datetime_field", None)
        self.assertEqual(value, None)

    def test_datetime_deserialization_required(self):
        field_name = "test_datetime_field"
        field = IDXTestDocumentSchema.get(field_name)
        with RequiredField(field):
            with self.assertRaises(RequiredMissing):
                self.deserialize(field_name, None)

    def test_text_deserialization_returns_decimal(self):
        value = self.deserialize("test_decimal_field", "1.1")
        self.assertTrue(isinstance(value, Decimal), "Not a <Decimal>")
        self.assertEqual(Decimal("1.1"), value)

    def test_dict_deserialization_returns_dict(self):
        value = self.deserialize("test_dict_field", {"key": "value"})
        self.assertTrue(isinstance(value, dict), "Not a <dict>")
        self.assertEqual({"key": "value"}, value)

    def test_float_deserialization_returns_float(self):
        value = self.deserialize("test_float_field", 1.0)
        self.assertTrue(isinstance(value, float), "Not a <float>")
        self.assertEqual(1.0, value)

    def test_frozenset_deserialization_returns_frozenset(self):
        value = self.deserialize("test_frozenset_field", ["foo", "bar"])
        self.assertTrue(isinstance(value, frozenset), "Not a <frozenset>")
        self.assertEqual(frozenset(["foo", "bar"]), value)

    def test_int_deserialization_returns_int(self):
        value = self.deserialize("test_int_field", 22)
        self.assertTrue(isinstance(value, int), "Not an <int>")
        self.assertEqual(22, value)

    def test_list_deserialization_returns_list(self):
        value = self.deserialize("test_list_field", [1, 2, 3])
        self.assertTrue(isinstance(value, list), "Not a <list>")
        self.assertEqual([1, 2, 3], value)

    def test_list_deserialization_from_tokens_returns_list_of_values(self):
        value = self.deserialize(
            "test_list_field_with_choice_with_vocabulary", ["token1", "token3"]
        )
        self.assertTrue(isinstance(value, list), "Not a <list>")
        self.assertEqual(["value1", "value3"], value)

    def test_list_deserialization_from_values_returns_list_of_values(self):
        value = self.deserialize(
            "test_list_field_with_choice_with_vocabulary", ["value1", "value3"]
        )
        self.assertTrue(isinstance(value, list), "Not a <list>")
        self.assertEqual(["value1", "value3"], value)

    def test_list_deserialization_from_terms_returns_list_of_values(self):
        value = self.deserialize(
            "test_list_field_with_choice_with_vocabulary",
            [
                {"token": "token1", "title": "title1"},
                {"token": "token3", "title": "title3"},
            ],
        )
        self.assertTrue(isinstance(value, list), "Not a <list>")
        self.assertEqual(["value1", "value3"], value)

    def test_set_deserialization_returns_set(self):
        value = self.deserialize("test_set_field", [1, 2, 3])
        self.assertTrue(isinstance(value, set), "Not a <set>")
        self.assertEqual({1, 2, 3}, value)

    def test_text_deserialization_returns_unicode(self):
        value = self.deserialize("test_text_field", "Foo")
        self.assertTrue(isinstance(value, str), "Not an <unicode>")
        self.assertEqual("Foo", value)

    def test_textline_deserialization_returns_unicode(self):
        value = self.deserialize("test_textline_field", "Foo")
        self.assertTrue(isinstance(value, str), "Not an <unicode>")
        self.assertEqual("Foo", value)

    def test_time_deserialization_returns_time(self):
        value = self.deserialize("test_time_field", "10:39:54.361Z")
        self.assertTrue(isinstance(value, time), "Not a <time>")
        self.assertEqual(time(10, 39, 54, 361000), value)

    def test_timedelta_deserialization_returns_timedela(self):
        value = self.deserialize("test_timedelta_field", 3600.0)
        self.assertTrue(isinstance(value, timedelta), "Not a <timedelta>")
        self.assertEqual(timedelta(seconds=3600), value)

    def test_tuple_deserialization_returns_tuple(self):
        value = self.deserialize("test_tuple_field", [1, 2, 3])
        self.assertTrue(isinstance(value, tuple), "Not a <tuple>")
        self.assertEqual((1, 2, 3), value)

    def test_nested_list_deserialization_returns_nested_list(self):
        value = self.deserialize("test_nested_list_field", [[1, "foo"], [2, "bar"]])
        self.assertTrue(isinstance(value, list), "Not a <list>")
        self.assertTrue(isinstance(value[0], tuple), "Not a <tuple>")
        self.assertTrue(isinstance(value[1], tuple), "Not a <tuple>")

    def test_nested_dict_deserialization_returns_nested_dict(self):
        value = self.deserialize(
            "test_nested_dict_field", {"1": ["foo", "bar"], "2": ["spam", "eggs"]}
        )
        self.assertTrue(isinstance(value, dict), "Not a <dict>")
        self.assertIn("1", value)
        self.assertTrue(isinstance(value["1"], tuple), "Not a <tuple>")
        self.assertIn("2", value)
        self.assertTrue(isinstance(value["2"], tuple), "Not a <tuple>")

    def test_richtext_deserialization_from_unicode_returns_richtext(self):
        value = self.deserialize("test_richtext_field", "<p>a paragraph</p>")
        self.assertTrue(isinstance(value, RichTextValue), "Not a <RichTextValue>")
        self.assertEqual("<p>a paragraph</p>", value.raw)

    def test_richtext_deserialization_from_dict_returns_richtext(self):
        value = self.deserialize("test_richtext_field", {"data": "Some text"})
        self.assertTrue(isinstance(value, RichTextValue), "Not a <RichTextValue>")
        self.assertEqual("Some text", value.raw)

    def test_richtext_deserialization_sets_mime_type(self):
        value = self.deserialize(
            "test_richtext_field",
            {"data": "Some text", "content-type": "text/plain"},
        )
        self.assertEqual("text/plain", value.mimeType)

    def test_richtext_deserialization_sets_encoding(self):
        value = self.deserialize(
            "test_richtext_field", {"data": "Some text", "encoding": "latin1"}
        )
        self.assertEqual("latin1", value.encoding)

    def test_richtext_deserialization_fix_apostrophe(self):
        value = self.deserialize("test_richtext_field", "<p>char with &#x27;</p>")
        self.assertEqual("<p>char with '</p>", value.raw)

    def test_namedfield_deserialization_decodes_value(self):
        value = self.deserialize(
            "test_namedfile_field",
            {"data": "U3BhbSBhbmQgZWdncyE=", "encoding": "base64"},
        )
        self.assertEqual(b"Spam and eggs!", value.data)

    def test_namedfield_deserialization_sets_content_type(self):
        value = self.deserialize(
            "test_namedfile_field",
            {"data": "Spam and eggs!", "content-type": "text/plain"},
        )
        self.assertEqual("text/plain", value.contentType)

    def test_namedfield_deserialization_sets_filename(self):
        value = self.deserialize(
            "test_namedfile_field",
            {"data": "Spam and eggs!", "filename": "doc.txt"},
        )
        self.assertEqual("doc.txt", value.filename)

    def test_namedfile_deserialization_returns_namedfile(self):
        value = self.deserialize("test_namedfile_field", {"data": "Spam and eggs!"})
        self.assertTrue(isinstance(value, namedfile.NamedFile), "Not a <NamedFile>")
        self.assertEqual(b"Spam and eggs!", value.data)

    def test_namedimage_deserialization_returns_namedimage(self):
        value = self.deserialize(
            "test_namedimage_field",
            {
                "data": "R0lGODlhAQABAIAAAP///wAAACwAAAAAAQABAAACAkQBADs=",
                "encoding": "base64",
                "content-type": "image/gif",
            },
        )
        self.assertTrue(isinstance(value, namedfile.NamedImage), "Not a <NamedImage>")
        self.assertTrue(value.data.startswith(b"GIF89a"))

    def test_namedblobfile_deserialization_returns_namedblobfile(self):
        value = self.deserialize("test_namedblobfile_field", {"data": "Spam and eggs!"})
        self.assertTrue(
            isinstance(value, namedfile.NamedBlobFile), "Not a <NamedBlobFile>"
        )
        self.assertEqual(b"Spam and eggs!", value.data)

    def test_namedblobimage_deserialization_returns_namedblobimage(self):
        value = self.deserialize(
            "test_namedblobimage_field",
            {
                "data": "R0lGODlhAQABAIAAAP///wAAACwAAAAAAQABAAACAkQBADs=",
                "encoding": "base64",
                "content-type": "image/gif",
            },
        )
        self.assertTrue(
            isinstance(value, namedfile.NamedBlobImage), "Not a <NamedBlobImage>"
        )
        self.assertTrue(value.data.startswith(b"GIF89a"))

    def test_namedblobimage_deserialization_fed_with_null_removes_image(self):
        # null in json translates to None in python.
        value = self.deserialize("test_namedblobimage_field", None)
        self.assertFalse(value)

    def test_namedblobfile_deserialization_fed_with_null_removes_file(self):
        # null in json translates to None in python.
        value = self.deserialize("test_namedblobfile_field", None)
        self.assertFalse(value)

    def test_namedblobfile_deserialize_required(self):
        field_name = "test_namedblobfile_field"
        field = IDXTestDocumentSchema.get(field_name)
        with RequiredField(field):
            with self.assertRaises(RequiredMissing):
                self.deserialize(field_name, None)

    def test_namedblobimage_deserialize_required(self):
        field_name = "test_namedblobimage_field"
        field = IDXTestDocumentSchema.get(field_name)
        with RequiredField(field):
            with self.assertRaises(RequiredMissing):
                self.deserialize(field_name, None)

    def test_relationchoice_deserialization_from_uid_returns_document(self):
        doc2 = self.portal[
            self.portal.invokeFactory(
                "DXTestDocument", id="doc2", title="Referenceable Document"
            )
        ]
        value = self.deserialize("test_relationchoice_field", str(doc2.UID()))
        self.assertEqual(doc2, value)

    def test_relationchoice_deserialization_from_url_returns_document(self):
        doc2 = self.portal[
            self.portal.invokeFactory(
                "DXTestDocument", id="doc2", title="Referenceable Document"
            )
        ]
        value = self.deserialize("test_relationchoice_field", str(doc2.absolute_url()))
        self.assertEqual(doc2, value)

    def test_relationchoice_deserialization_from_path_returns_document(self):
        doc2 = self.portal[
            self.portal.invokeFactory(
                "DXTestDocument", id="doc2", title="Referenceable Document"
            )
        ]
        value = self.deserialize("test_relationchoice_field", "/doc2")
        self.assertEqual(doc2, value)

    def test_relationchoice_deserialization_from_invalid_intid_raises(self):
        with self.assertRaises(ValueError) as cm:
            self.deserialize("test_relationchoice_field", 123456789)
        self.assertEqual(
            str(cm.exception), "Could not resolve object for intid=123456789"
        )
        self.assertEqual(400, self.request.response.getStatus())

    def test_relationchoice_deserialization_from_invalid_uid_raises(self):
        with self.assertRaises(ValueError) as cm:
            self.deserialize(
                "test_relationchoice_field",
                "ac12b24913cf45c6863937367aacc263",
            )
        self.assertEqual(
            str(cm.exception),
            "Could not resolve object for UID=ac12b24913cf45c6863937367aacc263",
        )
        self.assertEqual(400, self.request.response.getStatus())

    def test_relationchoice_deserialization_from_invalid_url_raises(self):
        with self.assertRaises(ValueError) as cm:
            self.deserialize(
                "test_relationchoice_field",
                "http://nohost/plone/doesnotexist",
            )
        self.assertEqual(
            str(cm.exception),
            "Could not resolve object for URL=http://nohost/plone/doesnotexist",
        )
        self.assertEqual(400, self.request.response.getStatus())

    def test_relationchoice_deserialization_from_invalid_path_raises(self):
        with self.assertRaises(ValueError) as cm:
            self.deserialize("test_relationchoice_field", "/doesnotexist")
        self.assertEqual(
            str(cm.exception), "Could not resolve object for path=/doesnotexist"
        )
        self.assertEqual(400, self.request.response.getStatus())

    def test_relationlist_deserialization_returns_list_of_documents(self):
        doc2 = self.portal[
            self.portal.invokeFactory(
                "DXTestDocument", id="doc2", title="Referenceable Document"
            )
        ]
        doc3 = self.portal[
            self.portal.invokeFactory(
                "DXTestDocument", id="doc3", title="Referenceable Document"
            )
        ]
        value = self.deserialize(
            "test_relationlist_field",
            [str(doc2.UID()), str(doc3.UID())],
        )
        self.assertTrue(isinstance(value, list), "Not a <list>")
        self.assertEqual(doc2, value[0])
        self.assertEqual(doc3, value[1])

    def test_default_deserializer_validates_value(self):
        with self.assertRaises(ValidationError):
            self.deserialize("test_maxlength_field", "01234567890")

    def test_datetime_deserializer_handles_invalid_value(self):
        with self.assertRaises(ValueError) as cm:
            self.deserialize("test_datetime_field", "2015-15-15T10:39:54.361Z")
        self.assertEqual("Invalid date: 2015-15-15T10:39:54.361Z", str(cm.exception))

    def test_datetime_deserializer_validates_value(self):
        with self.assertRaises(ValidationError):
            self.deserialize("test_datetime_min_field", "1999-12-20T10:39:54.361Z")

    def test_collection_deserializer_validates_value(self):
        with self.assertRaises(ValidationError) as cm:
            self.deserialize("test_list_value_type_field", [1, b"2", 3])

        # This validation error is actually produced by the
        # DefaultFieldDeserializer that the CollectionFieldDeserializer will
        # delegate to for deserializing collection items.
        self.assertEqual("Object is of wrong type.", cm.exception.doc())
        self.assertEqual((b"2", (int,), ""), cm.exception.args)

    def test_dict_deserializer_validates_value(self):
        with self.assertRaises(ValidationError) as cm:
            self.deserialize("test_dict_key_type_field", {"k": "v"})

        # This validation error is actually produced by the
        # DefaultFieldDeserializer that the DictFieldSerializer will delegate
        # to for deserializing keys and values.
        # We check for two sets of exception details
        # because zope.schema changed its exception...
        self.assertIn(
            cm.exception.doc(), ("Object is of wrong type.", "Invalid int literal.")
        )
        self.assertIn(
            cm.exception.args,
            (
                ("k", (int,), ""),
                ("invalid literal for int() with base 10: 'k'",),
            ),
        )

    def test_time_deserializer_handles_invalid_value(self):
        with self.assertRaises(ValueError) as cm:
            self.deserialize("test_time_field", "midnight")
        self.assertEqual("Invalid time: midnight", str(cm.exception))

    def test_time_deserializer_validates_value(self):
        with self.assertRaises(ValidationError) as cm:
            self.deserialize("test_time_min_field", "00:39:54.361Z")
        self.assertEqual("Value is too small", cm.exception.doc())

    def test_timedelta_deserializer_handles_invalid_value(self):
        with self.assertRaises(ValueError) as cm:
            self.deserialize("test_timedelta_field", "2h")
        self.assertIn(
            "unsupported type for timedelta seconds component:", str(cm.exception)
        )

    def test_timedelta_deserializer_validates_value(self):
        with self.assertRaises(ValidationError) as cm:
            self.deserialize("test_timedelta_min_field", 50)
        self.assertEqual("Value is too small", cm.exception.doc())

    def test_namedfield_deserializer_validates_value(self):
        with self.assertRaises(ValidationError) as cm:
            self.deserialize(
                "test_namedimage_field",
                {"data": "Spam and eggs!", "content-type": "text/plain"},
            )
        self.assertEqual("Invalid image file", cm.exception.doc())

    def test_namedfield_deserializer_download(self):
        # Handle when we post back the GET results.
        # This then has a 'download' key, and not a 'data' key.

        self.deserialize(
            "test_namedfile_field",
            {"download": "some download link", "content-type": "text/plain"},
        )

    def test_richtextfield_deserializer_validates_value(self):
        with self.assertRaises(ValidationError) as cm:
            self.deserialize(
                "test_richtext_field",
                {"data": "Spam and eggs!", "content-type": "text/xml"},
            )
        self.assertEqual("Object is of wrong type.", cm.exception.doc())

    def test_relationchoicefield_deserializer_validates_value(self):
        self.portal[
            self.portal.invokeFactory(
                "DXTestDocument", id="doc3", title="Referenceable Document"
            )
        ]
        with self.assertRaises(ValidationError) as cm:
            self.deserialize("test_relationchoice_field", "/doc3")
        self.assertEqual("Constraint not satisfied", cm.exception.doc())

    def test_deserialize_with_context_bound_vocabulary(self):
        value = self.deserialize(
            "test_list_choice_with_context_vocabulary_field", ["portal_catalog"]
        )
        self.assertEqual(["portal_catalog"], value)

    def test_textline_deserializer_strips_value(self):
        value = self.deserialize("test_textline_field", "  aa  ")
        self.assertEqual(value, "aa")

    def test_default_field_deserializer_validates_value(self):
        class CustomIntField(Field):
            def constraint(self, value):
                if not isinstance(value, int):
                    raise ConstraintNotSatisfied
                return True

        field = CustomIntField()
        deserializer = getMultiAdapter(
            (field, self.portal.doc1, self.request), IFieldDeserializer
        )

        with self.assertRaises(ConstraintNotSatisfied):
            deserializer(b"not an int")

        self.assertEqual(42, deserializer(42))

    def test_textline_deserializer_for_links_convert_internal_links(self):
        self.portal.invokeFactory("Link", id="link", title="Test Link")
        link = self.portal.link
        field = None
        for schema in iterSchemata(link):
            if "remoteUrl" in schema:
                field = schema.get("remoteUrl")
                break
        deserializer = getMultiAdapter((field, link, self.request), IFieldDeserializer)
        doc_uuid = IUUID(self.portal.doc1)

        self.assertEqual(
            "http://www.plone.com", deserializer(value="http://www.plone.com")
        )
        self.assertEqual(
            f"../resolveuid/{doc_uuid}", deserializer(value="http://nohost/plone/doc1")
        )

        # I want to save internal URLs as resolveuid
        self.assertEqual(f"../resolveuid/{doc_uuid}", deserializer(value="/doc1"))

        # If ${portal_url} is present, leave it as it is
        self.assertEqual("${portal_url}/doc1", deserializer(value="${portal_url}/doc1"))

        # for other contents/fields does nothing
        value = self.deserialize("test_textline_field", "http://www.plone.com")
        self.assertEqual("http://www.plone.com", value)
        value = self.deserialize("test_textline_field", "http://nohost/plone/doc1")
        self.assertEqual(self.portal.doc1.absolute_url(), value)



from DateTime import DateTime
from plone.restapi.deserializer import json_body
from plone.restapi.interfaces import IDeserializeFromJson
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.services import Service
from plone.restapi.services.workflow.utils import elevated_privileges
from Products.CMFCore.interfaces import IFolderish
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException
from zExceptions import BadRequest
from zope.component import queryMultiAdapter
from zope.i18n import translate
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse
from zope.publisher.interfaces import NotFound

import plone.protect.interfaces


@implementer(IPublishTraverse)
class WorkflowTransition(Service):
    """Trigger workflow transition"""

    def __init__(self, context, request):
        super().__init__(context, request)
        self.transition = None
        self.wftool = getToolByName(context, "portal_workflow")

    def publishTraverse(self, request, name):
        if self.transition is None:
            self.transition = name
        else:
            raise NotFound(self, name, request)
        return self

    def reply(self):
        if self.transition is None:
            self.request.response.setStatus(400)
            return dict(error=dict(type="BadRequest", message="Missing transition"))

        data = json_body(self.request)

        # Disable CSRF protection
        if "IDisableCSRFProtection" in dir(plone.protect.interfaces):
            alsoProvides(self.request, plone.protect.interfaces.IDisableCSRFProtection)

        comment = data.get("comment", "")
        include_children = data.get("include_children", False)
        publication_dates = {}
        if "effective" in data:
            publication_dates["effective"] = data["effective"]
        if "expires" in data:
            publication_dates["expires"] = data["expires"]
        # Archetypes has different field names
        if "effectiveDate" in data:
            publication_dates["effectiveDate"] = data["effectiveDate"]
        if "expirationDate" in data:
            publication_dates["expirationDate"] = data["expirationDate"]

        try:
            self.recurse_transition(
                [self.context], comment, publication_dates, include_children
            )

        except WorkflowException as e:
            self.request.response.setStatus(400)
            return dict(
                error=dict(
                    type="WorkflowException",
                    message=translate(str(e), context=self.request),
                )
            )
        except BadRequest as e:
            self.request.response.setStatus(400)
            return dict(error=dict(type="Bad Request", message=str(e)))

        with elevated_privileges(self.context):
            try:
                history = self.wftool.getInfoFor(self.context, "review_history")
                action = history[-1]
                action["title"] = self.context.translate(
                    self.wftool.getTitleForStateOnType(
                        action["review_state"], self.context.portal_type
                    )
                )
            except WorkflowException as e:
                self.request.response.setStatus(400)
                action = dict(
                    error=dict(
                        type="WorkflowException",
                        message=translate(str(e), context=self.request),
                    )
                )

        return json_compatible(action)

    def recurse_transition(
        self, objs, comment, publication_dates, include_children=False
    ):
        for obj in objs:
            if publication_dates:
                deserializer = queryMultiAdapter(
                    (obj, self.request), IDeserializeFromJson
                )
                deserializer(data=publication_dates)

            if obj.EffectiveDate() == "None":
                obj.setEffectiveDate(DateTime())
                obj.reindexObject()
            if not self.wftool.getWorkflowsFor(obj):
                continue
            try:
                self.wftool.doActionFor(obj, self.transition, comment=comment)
            except WorkflowException as e:
                if not self.is_same_state(obj):
                    # this is a real error
                    raise e
            if include_children and IFolderish.providedBy(obj):
                self.recurse_transition(
                    obj.objectValues(), comment, publication_dates, include_children
                )

    def is_same_state(self, obj):
        """
        Return True if the object is already in the transition's destination state.
        """
        review_state = self.wftool.getInfoFor(ob=obj, name="review_state")

        for wf in self.wftool.getWorkflowsFor(obj):
            for transition in wf.transitions.objectValues():
                if (
                    review_state == transition.new_state_id
                    and self.transition == transition.id
                ):
                    return True
        return False



# XXX: EXPERIMENTAL!!!
# This is an experimental feature meant for use in Volto only!
# This code is likely to change in the future, even within minor releases.
# We will make sure plone.restapi latest always works with the latest Volto release.
# This code is planned to being refactored into plone.volto before CMFPlone 6.0 is out.
# <tisto@plone.org>

from plone.app.contenttypes.indexers import SearchableText
from plone.indexer.decorator import indexer
from plone.restapi.behaviors import IBlocks
from plone.restapi.interfaces import IBlockSearchableText
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.publisher.interfaces.browser import IBrowserRequest


@implementer(IBlockSearchableText)
@adapter(IBlocks, IBrowserRequest)
class TextBlockSearchableText:
    """Searchable Text indexer for Text (DraftJS) blocks."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, value):
        result = ""
        for paragraph in value.get("text", {}).get("blocks", {}):
            text = paragraph["text"]
            result = " ".join((result, text))
        return result


@implementer(IBlockSearchableText)
@adapter(IBlocks, IBrowserRequest)
class TableBlockSearchableText:
    """Searchable Text indexer for Table blocks."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, value):
        table = value.get("table", {})
        if not table:
            return ""
        result = ""
        for row in table.get("rows", []):
            for cell in row.get("cells", []):
                for paragraph in cell.get("value", {}).get("blocks", {}):
                    text = paragraph["text"]
                    result = " ".join((result, text))
        return result


@implementer(IBlockSearchableText)
@adapter(IBlocks, IBrowserRequest)
class SlateTextIndexer:
    """Searchable Text indexer for Slate blocks."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, block):
        block = block or {}
        # BBB compatibility with slate blocks that used the "plaintext" field
        return block.get("plaintext", "")


def extract_subblocks(block):
    """Extract subblocks from a block.

    :param block: Dictionary with block information.
    :returns: A list with subblocks, if present, or an empty list.
    """
    if "data" in block and "blocks" in block["data"]:
        raw_blocks = block["data"]["blocks"]
    elif "blocks" in block:
        raw_blocks = block["blocks"]
    else:
        raw_blocks = None
    return list(raw_blocks.values()) if isinstance(raw_blocks, dict) else []


def extract_text(block, obj, request):
    """Extract text information from a block.

    This function tries the following methods, until it finds a result:
        1. searchableText attribute
        2. Server side adapter
        3. Subblocks

    The decision to use the server side adapter before the subblocks traversal
    allows addon developers to choose this implementation when they want a
    more granular control of the indexing.

    :param block: Dictionary with block information.
    :param obj: Context to be used to get a IBlockSearchableText.
    :param request: Current request.
    :returns: A string with text found in the block.
    """
    result = ""
    block_type = block.get("@type", "")
    # searchableText is the conventional way of storing
    # searchable info in a block
    searchableText = block.get("searchableText", "")
    if searchableText:
        # TODO: should we evaluate in some way this value? maybe passing
        # it into html/plain text transformer?
        return searchableText
    # Use server side adapters to extract the text data
    adapter = queryMultiAdapter((obj, request), IBlockSearchableText, name=block_type)
    result = adapter(block) if adapter is not None else ""
    if not result:
        subblocks = extract_subblocks(block)
        for subblock in subblocks:
            tmp_result = extract_text(subblock, obj, request)
            result = f"{result}\n{tmp_result}"
    return result


@indexer(IBlocks)
def SearchableText_blocks(obj):
    """Extract text to be used by the SearchableText index in the Catalog."""
    request = getRequest()
    blocks = obj.blocks
    blocks_layout = obj.blocks_layout
    blocks_text = []
    for block_id in blocks_layout.get("items", []):
        block = blocks.get(block_id, {})
        blocks_text.append(extract_text(block, obj, request))

    # Extract text using the base plone.app.contenttypes indexer
    std_text = SearchableText(obj)
    blocks_text.append(std_text)
    return " ".join([text.strip() for text in blocks_text if text.strip()])



from plone.restapi.exceptions import DeserializationError

import json


def json_body(request):
    try:
        data = json.loads(request.get("BODY") or "{}")
    except ValueError:
        raise DeserializationError("No JSON object could be decoded")
    if not isinstance(data, dict):
        raise DeserializationError("Malformed body")
    return data


def boolean_value(value):
    """

    Args:
        value: a value representing a boolean which can be
               a string, a boolean or an integer
                   (usually a string from a GET parameter).

    Returns: a boolean

    """
    return value not in {False, "false", "False", "0", 0}



from zExceptions import BadRequest

import json


class OrderingMixin:
    # This assumes there is a self.folder with 9 documents in it.

    def test_reorder(self):
        # We run all this in one test, because of dependend ordering.
        # initial situation
        self.assertEqual(
            [
                "doc1",
                "doc2",
                "doc3",
                "doc4",
                "doc5",
                "doc6",
                "doc7",
                "doc8",
                "doc9",
            ],  # noqa
            self.folder.contentIds(),
        )

        # Normal
        # Move to top
        data = {"ordering": {"delta": "top", "obj_id": "doc9"}}
        self.deserialize(body=json.dumps(data), context=self.folder)
        self.assertEqual(
            [
                "doc9",
                "doc1",
                "doc2",
                "doc3",
                "doc4",
                "doc5",
                "doc6",
                "doc7",
                "doc8",
            ],  # noqa
            self.folder.contentIds(),
        )

        # Move to bottom
        data = {"ordering": {"delta": "bottom", "obj_id": "doc9"}}
        self.deserialize(body=json.dumps(data), context=self.folder)
        self.assertEqual(
            [
                "doc1",
                "doc2",
                "doc3",
                "doc4",
                "doc5",
                "doc6",
                "doc7",
                "doc8",
                "doc9",
            ],  # noqa
            self.folder.contentIds(),
        )

        # Delta up
        data = {"ordering": {"delta": -2, "obj_id": "doc5"}}
        self.deserialize(body=json.dumps(data), context=self.folder)
        self.assertEqual(
            [
                "doc1",
                "doc2",
                "doc5",
                "doc3",
                "doc4",
                "doc6",
                "doc7",
                "doc8",
                "doc9",
            ],  # noqa
            self.folder.contentIds(),
        )

        # Delta down
        data = {"ordering": {"delta": 2, "obj_id": "doc6"}}
        self.deserialize(body=json.dumps(data), context=self.folder)
        self.assertEqual(
            [
                "doc1",
                "doc2",
                "doc5",
                "doc3",
                "doc4",
                "doc7",
                "doc8",
                "doc6",
                "doc9",
            ],  # noqa
            self.folder.contentIds(),
        )

        # subset ids
        # Move to top
        data = {
            "ordering": {
                "delta": "top",
                "obj_id": "doc8",
                "subset_ids": ["doc2", "doc3", "doc8"],
            }
        }  # noqa
        self.deserialize(body=json.dumps(data), context=self.folder)
        self.assertEqual(
            [
                "doc1",
                "doc8",
                "doc5",
                "doc2",
                "doc4",
                "doc7",
                "doc3",
                "doc6",
                "doc9",
            ],  # noqa
            self.folder.contentIds(),
        )

        # Move to bottom
        data = {
            "ordering": {
                "delta": "bottom",
                "obj_id": "doc8",
                "subset_ids": ["doc8", "doc2", "doc3"],
            }
        }  # noqa
        self.deserialize(body=json.dumps(data), context=self.folder)
        self.assertEqual(
            [
                "doc1",
                "doc2",
                "doc5",
                "doc3",
                "doc4",
                "doc7",
                "doc8",
                "doc6",
                "doc9",
            ],  # noqa
            self.folder.contentIds(),
        )

        # Delta up
        data = {
            "ordering": {
                "delta": -1,
                "obj_id": "doc8",
                "subset_ids": ["doc2", "doc3", "doc8"],
            }
        }  # noqa
        self.deserialize(body=json.dumps(data), context=self.folder)
        self.assertEqual(
            [
                "doc1",
                "doc2",
                "doc5",
                "doc8",
                "doc4",
                "doc7",
                "doc3",
                "doc6",
                "doc9",
            ],  # noqa
            self.folder.contentIds(),
        )

        # Delta down
        data = {
            "ordering": {
                "delta": 1,
                "obj_id": "doc2",
                "subset_ids": ["doc2", "doc8", "doc3"],
            }
        }  # noqa
        self.deserialize(body=json.dumps(data), context=self.folder)
        self.assertEqual(
            [
                "doc1",
                "doc8",
                "doc5",
                "doc2",
                "doc4",
                "doc7",
                "doc3",
                "doc6",
                "doc9",
            ],  # noqa
            self.folder.contentIds(),
        )

    def test_ordering_preserves_native_string_obj_id(self):
        # sanity check, initial situation
        for id_ in self.folder.objectIds():
            self.assertIsInstance(id_, str)

        # reorder
        data = {"ordering": {"delta": "top", "obj_id": "doc9"}}
        self.deserialize(body=json.dumps(data), context=self.folder)

        # reordering should preserve bytestring ids in PY2 and unicode ids in PY3
        for id_ in self.folder.objectIds():
            self.assertIsInstance(id_, str)

    def test_ordering_preserves_native_string_subset_ids(self):
        # sanity check, initial situation
        for id_ in self.folder.objectIds():
            self.assertIsInstance(id_, str)

        # reorder and also provide subset_ids
        data = {
            "ordering": {
                "delta": "bottom",
                "obj_id": "doc1",
                "subset_ids": ["doc1", "doc2", "doc3"],
            }
        }  # noqa
        self.deserialize(body=json.dumps(data), context=self.folder)

        # reordering should preserve bytestring ids in PY2 and unicode ids in PY3
        for id_ in self.folder.objectIds():
            self.assertIsInstance(id_, str)

    def test_reorder_subsetids(self):
        # sanity check, initial situation
        self.assertEqual(
            [
                "doc1",
                "doc2",
                "doc3",
                "doc4",
                "doc5",
                "doc6",
                "doc7",
                "doc8",
                "doc9",
            ],  # noqa
            self.folder.contentIds(),
        )

        data = {
            "ordering": {
                "delta": 1,
                "obj_id": "doc8",
                "subset_ids": ["doc2", "doc8", "doc6"],
            }
        }  # noqa

        with self.assertRaises(BadRequest) as cm:
            self.deserialize(body=json.dumps(data), context=self.folder)

        self.assertEqual("Client/server ordering mismatch", str(cm.exception))

    def test_resort_all_items(self):
        self.assertEqual(
            [
                "doc1",
                "doc2",
                "doc3",
                "doc4",
                "doc5",
                "doc6",
                "doc7",
                "doc8",
                "doc9",
            ],  # noqa
            self.folder.contentIds(),
        )

        # Resort all Ids descending
        data = {"sort": {"on": "id", "order": "descending"}}
        self.deserialize(body=json.dumps(data), context=self.folder)
        self.assertEqual(
            [
                "doc9",
                "doc8",
                "doc7",
                "doc6",
                "doc5",
                "doc4",
                "doc3",
                "doc2",
                "doc1",
            ],  # noqa
            self.folder.contentIds(),
        )



from DateTime import DateTime
from plone.restapi.interfaces import IIndexQueryParser
from plone.restapi.search.query import BaseIndexQueryParser
from Products.DateRecurringIndex.index import DateRecurringIndex
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(IIndexQueryParser)
@adapter(DateRecurringIndex, Interface, Interface)
class DateRecurringIndexQueryParser(BaseIndexQueryParser):

    query_value_type = DateTime
    query_options = {"range": str}



from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from Products.CMFCore.interfaces._tools import IMemberData
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from zope.component import adapter
from zope.component.hooks import getSite
from zope.interface import implementer
from zope.publisher.interfaces import IRequest
from zope.schema import getFieldNames
from plone.app.users.browser.userdatapanel import getUserDataSchema


class BaseSerializer:
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        user = self.context
        portal = getSite()

        # Global roles
        roles = user.getRoles()
        # Anonymous and Authenticated are pseudo roles assign automatically
        # to logged-in or logged-out users. They should not be exposed here
        roles = list(set(roles) - {"Anonymous", "Authenticated"})

        data = {
            "@id": f"{portal.absolute_url()}/@users/{user.id}",
            "id": user.id,
            "username": user.getUserName(),
            "roles": roles,
        }

        schema = getUserDataSchema()

        for name in getFieldNames(schema):
            if name == "portrait":
                memberdata = getToolByName(portal, "portal_memberdata")
                if user.id in memberdata.portraits:
                    value = "{}/portal_memberdata/portraits/{}".format(
                        portal.absolute_url(), user.id
                    )
                else:
                    value = None
            elif name == "pdelete":
                continue
            else:
                value = user.getProperty(name, "")
                if value == "":
                    value = None
                if value:
                    value = safe_unicode(value)
            data[name] = value

        return data


@implementer(ISerializeToJson)
@adapter(IMemberData, IRequest)
class SerializeUserToJson(BaseSerializer):
    pass


@implementer(ISerializeToJsonSummary)
@adapter(IMemberData, IRequest)
class SerializeUserToJsonSummary(BaseSerializer):
    pass



from datetime import datetime
from plone.app.textfield.value import RichTextValue
from plone.dexterity.interfaces import IDexterityContent
from plone.namedfile.file import NamedBlobImage
from Products.CMFCore.utils import getToolByName
from zope.component.hooks import getSite

import logging
import os
import pytz


logger = logging.getLogger("plone.restapi.performance")

LOREMIPSUM_HTML_10_PARAGRAPHS = """<p>Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.</p><p>Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi. Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat.</p><p>Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi.</p><p>Nam liber tempor cum soluta nobis eleifend option congue nihil imperdiet doming id quod mazim placerat facer possim assum. Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat.</p><p>Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis.</p><p>At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, At accusam aliquyam diam diam dolore dolores duo eirmod eos erat, et nonumy sed tempor et et invidunt justo labore Stet clita ea et gubergren, kasd magna no rebum. sanctus sea sed takimata ut vero voluptua. est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat.</p><p>Consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus.</p><p>Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.</p><p>Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi. Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat.</p><p>Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi."""  # noqa

LOREMIPSUM_TEXT_PARAGRAPH = "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum."  # noqa


def set_description(obj):
    if IDexterityContent.providedBy(obj):
        obj.description = LOREMIPSUM_TEXT_PARAGRAPH
    else:
        obj.setDescription(LOREMIPSUM_TEXT_PARAGRAPH)


def set_text(obj):
    if IDexterityContent.providedBy(obj):
        obj.text = RichTextValue(
            LOREMIPSUM_HTML_10_PARAGRAPHS, "text/html", "text/x-html-safe"
        )
    else:
        obj.setText(LOREMIPSUM_HTML_10_PARAGRAPHS)


def set_image(obj):
    if IDexterityContent.providedBy(obj):
        from plone.namedfile.file import NamedBlobImage

        filename = os.path.join(os.path.dirname(__file__), "image.png")
        obj.image = NamedBlobImage(data=open(filename, "rb").read(), filename=filename)
    else:
        filename = os.path.join(os.path.dirname(__file__), "image.png")
        obj.setImage(open(filename, "rb").read())


def set_file(obj):
    if IDexterityContent.providedBy(obj):
        from plone.namedfile.file import NamedBlobFile

        filename = os.path.join(os.path.dirname(__file__), "file.pdf")
        obj.file = NamedBlobFile(
            data=open(filename, "rb").read(),
            filename=filename,
            contentType="application/pdf",
        )
    else:
        filename = os.path.join(os.path.dirname(__file__), "file.pdf")
        obj.setFile(open(filename, "rb").read())


def publish(content):
    """Publish the object if it hasn't been published."""
    portal_workflow = getToolByName(getSite(), "portal_workflow")
    if portal_workflow.getInfoFor(content, "review_state") != "published":
        portal_workflow.doActionFor(content, "publish")
        return True
    return False


def step_setup_content(context):
    marker_file = "plone.restapi_performance_testing.txt"
    if context.readDataFile(marker_file) is None:
        return
    portal = getSite()

    # Testfolder WRITE
    portal.invokeFactory("Folder", id="testfolder-write", title="Testfolder Write")
    publish(portal["testfolder-write"])

    # Testfolder READ
    portal.invokeFactory("Folder", id="testfolder-read", title="Testfolder Read")
    publish(portal["testfolder-read"])
    portal = portal["testfolder-read"]

    # Document
    portal.invokeFactory("Document", id="document", title="Document")
    portal.document.description = LOREMIPSUM_TEXT_PARAGRAPH
    set_description(portal.document)
    set_text(portal.document)
    publish(portal.document)
    portal.document.reindexObject()

    # News Item
    portal.invokeFactory("News Item", id="newsitem", title="NewsItem")
    set_description(portal.newsitem)
    set_text(portal.newsitem)
    set_image(portal.newsitem)
    publish(portal.newsitem)
    portal.newsitem.reindexObject()

    # Folder
    portal.invokeFactory("Folder", id="folder", title="Folder")
    set_description(portal.folder)
    publish(portal.folder)

    # Folder with 10 Items
    logger.info("Creating 10 items in folder")
    portal.invokeFactory("Folder", id="folder-with-10-items", title="Folder 10")
    folder10 = portal["folder-with-10-items"]
    set_description(folder10)
    publish(folder10)
    for i in range(1, 11):
        folder10.invokeFactory("Document", id=f"doc{i}", title=f"Doc {i}")
        publish(folder10[f"doc{i}"])

    # Folder with 100 Items
    logger.info("Creating folder with 100 items")
    portal.invokeFactory("Folder", id="folder-with-100-items", title="Folder 100")
    folder100 = portal["folder-with-100-items"]
    set_description(folder100)
    publish(folder100)
    for i in range(1, 101):
        folder100.invokeFactory("Document", id=f"doc{i}", title=f"Doc {i}")
        publish(folder100[f"doc{i}"])

    # Folder with 1000 Items
    logger.info("Creating folder with 1000 items")
    portal.invokeFactory("Folder", id="folder-with-1000-items", title="Folder 1000")
    folder1000 = portal["folder-with-1000-items"]
    set_description(folder1000)
    publish(folder1000)
    for i in range(1, 1001):
        folder1000.invokeFactory("Document", id=f"doc{i}", title=f"Doc {i}")
        publish(folder1000[f"doc{i}"])

    # Folder with 10k Items
    logger.info("Creating folder with 10k items")
    portal.invokeFactory("Folder", id="folder-with-10k-items", title="Folder 10k")
    folder10k = portal["folder-with-10k-items"]
    set_description(folder10k)
    publish(folder10k)
    for i in range(1, 10001):
        folder10k.invokeFactory("Document", id=f"doc{i}", title=f"Doc {i}")
        publish(folder10k[f"doc{i}"])

    # # Folder with 100k Items
    # logger.info("Creating folder with 100k items")
    # portal.invokeFactory("Folder", id="folder-with-100k-items", title="Folder 100k")
    # folder100k = portal["folder-with-100k-items"]
    # set_description(folder100k)
    # publish(folder100k)
    # for i in range(1, 100001):
    #     folder100k.invokeFactory("Document", id=f"doc{i}", title=f"Doc {i}")
    #     publish(folder100k[f"doc{i}"])

    # Folder with 10 Items and next/previous enabled
    portal.invokeFactory(
        "Folder",
        id="folder-with-10-items-next-prev-enabled",
        title="Folder 10 (next/prev enabled)",
    )
    folder10np = portal["folder-with-10-items-next-prev-enabled"]
    folder10np.nextPreviousEnabled = True
    set_description(folder10np)
    publish(folder10np)
    for i in range(1, 11):
        folder10np.invokeFactory("Document", id=f"doc{i}", title=f"Doc {i}")
        publish(folder10np[f"doc{i}"])

    # Collection
    portal.invokeFactory("Collection", id="collection", title="Collection")
    set_description(portal.collection)
    set_text(portal.collection)
    publish(portal.collection)
    portal.collection.reindexObject()

    # Collection with Items
    portal.invokeFactory(
        "Collection", id="collectionitems", title="Collection with Items"
    )
    set_description(portal.collectionitems)
    set_text(portal.collectionitems)
    if IDexterityContent.providedBy(portal.collectionitems):
        portal.collectionitems.query = [
            {
                "i": "Type",
                "o": "plone.app.querystring.operation.string.is",
                "v": "Document",
            }
        ]
    publish(portal.collectionitems)
    portal.collectionitems.reindexObject()

    # Event
    portal.invokeFactory("Event", id="event", title="Event")
    set_description(portal.event)
    publish(portal.event)
    if IDexterityContent.providedBy(portal.event):
        portal.event.timezone = "Europe/Vienna"
        tz = pytz.timezone("Europe/Vienna")
        portal.event.start = tz.localize(datetime(2010, 10, 10, 12, 12))
        portal.event.end = tz.localize(datetime(2010, 10, 10, 13, 13))

    # Link
    portal.invokeFactory("Link", id="link", title="Link")
    set_description(portal.link)
    portal.link.remoteUrl = "http://plone.org"
    publish(portal.link)

    # Image
    portal.invokeFactory("Image", id="image", title="Image")
    set_description(portal.image)
    set_image(portal.image)

    # File
    portal.invokeFactory("File", id="file", title="File")
    set_description(portal.file)
    set_file(portal.file)

    # Image 1 MB
    portal.invokeFactory("Image", id="image-1mb", title="Image 1 MB")
    filename = os.path.join(os.path.dirname(__file__), "images", "image-1mb.jpg")
    portal.get("image-1mb").image = NamedBlobImage(
        data=open(filename, "rb").read(), filename=filename
    )

    # Image 2 MB
    portal.invokeFactory("Image", id="image-2mb", title="Image 2 MB")
    filename = os.path.join(os.path.dirname(__file__), "images", "image-2mb.jpg")
    portal.get("image-2mb").image = NamedBlobImage(
        data=open(filename, "rb").read(), filename=filename
    )

    # Image 3 MB
    portal.invokeFactory("Image", id="image-3mb", title="Image 3 MB")
    filename = os.path.join(os.path.dirname(__file__), "images", "image-3mb.jpg")
    portal.get("image-3mb").image = NamedBlobImage(
        data=open(filename, "rb").read(), filename=filename
    )

    # Image 10 MB
    portal.invokeFactory("Image", id="image-10mb", title="Image 10 MB")
    filename = os.path.join(os.path.dirname(__file__), "images", "image-10mb.jpg")
    portal.get("image-10mb").image = NamedBlobImage(
        data=open(filename, "rb").read(), filename=filename
    )

    # Volto Page with images and news items
    portal.invokeFactory("Folder", id="volto-page", title="Volto Page")
    volto_page = portal.get("volto-page")
    publish(volto_page)

    for i in range(1, 31):
        volto_page.invokeFactory("News Item", id=f"newsitem{i}", title=f"NewsItem {i}")
        newsitem = volto_page.get(f"newsitem{i}")
        set_description(newsitem)
        set_text(newsitem)
        image_file = os.path.join(os.path.dirname(__file__), "image.jpeg")
        with open(image_file, "rb") as f:
            image_data = f.read()
        newsitem.image = NamedBlobImage(
            data=image_data, contentType="image/jpeg", filename="image.jpeg"
        )
        publish(newsitem)
        newsitem.reindexObject()

    for i in range(1, 31):
        volto_page.invokeFactory("Image", id=f"image{i}", title=f"Image {i}")
        image_file = os.path.join(os.path.dirname(__file__), "image.jpeg")
        with open(image_file, "rb") as f:
            image_data = f.read()
        volto_page.get(f"image{i}").image = NamedBlobImage(
            data=image_data, contentType="image/jpeg", filename="image.jpeg"
        )

    # Volto
    # portal.invokeFactory("Document", id="volto", title="Volto")
    # portal.volto.description = LOREMIPSUM_TEXT_PARAGRAPH
    volto_page.blocks = {
        "0d57ccc7-8ceb-4b8f-bf95-5919eb859925": {
            "@type": "text",
            "text": {
                "blocks": [
                    {
                        "data": {},
                        "depth": 0,
                        "entityRanges": [],
                        "inlineStyleRanges": [],
                        "key": "4q33c",
                        "text": "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. ",
                        "type": "unstyled",
                    }
                ],
                "entityMap": {},
            },
        },
        "2172b929-70de-416b-b345-b98916a9726f": {
            "@type": "image",
            "alt": "jamie-street-PzGiZBpv72Y-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/jamie-street-pzgizbpv72y-unsplash.jpg",
        },
        "23dd130c-d90e-48d8-ab5a-9fb85a2136a4": {
            "@type": "image",
            "alt": "anastasia-petrova-xu2WYJek5AI-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/anastasia-petrova-xu2wyjek5ai-unsplash.jpg",
        },
        "23e99d38-da30-441b-8713-bbbad4344a85": {
            "@type": "image",
            "alt": "matt-artz-oSly6rQh7YA-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/matt-artz-osly6rqh7ya-unsplash.jpg",
        },
        "24b5b799-d755-4e43-97f5-e240d6b3b7df": {
            "@type": "image",
            "alt": "christopher-windus-ys_PVhkEC6c-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/christopher-windus-ys_pvhkec6c-unsplash.jpg",
        },
        "28b1c856-4ecd-4c43-bce5-9be127f29a34": {
            "@type": "text",
            "text": {
                "blocks": [
                    {
                        "data": {},
                        "depth": 0,
                        "entityRanges": [],
                        "inlineStyleRanges": [],
                        "key": "c0mrv",
                        "text": "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. ",
                        "type": "unstyled",
                    }
                ],
                "entityMap": {},
            },
        },
        "2e005fdb-a109-4148-b78c-855dab1e83a9": {
            "@type": "text",
            "text": {
                "blocks": [
                    {
                        "data": {},
                        "depth": 0,
                        "entityRanges": [],
                        "inlineStyleRanges": [],
                        "key": "b6u11",
                        "text": "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. ",
                        "type": "unstyled",
                    }
                ],
                "entityMap": {},
            },
        },
        "367aacb2-8124-43ce-8989-c9394b71ba97": {
            "@type": "image",
            "alt": "hector-j-rivas-QNc9tTNHRyI-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/hector-j-rivas-qnc9ttnhryi-unsplash.jpg",
        },
        "374bcb3f-d6c7-4ec0-9a26-aa9ee148a22a": {
            "@type": "text",
            "text": {
                "blocks": [
                    {
                        "data": {},
                        "depth": 0,
                        "entityRanges": [],
                        "inlineStyleRanges": [],
                        "key": "6o8vv",
                        "text": "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. ",
                        "type": "unstyled",
                    }
                ],
                "entityMap": {},
            },
        },
        "38076104-d93a-4e25-b2b6-7917be41dd48": {
            "@type": "image",
            "alt": "jukan-tateisi-bJhT_8nbUA0-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/jukan-tateisi-bjht_8nbua0-unsplash.jpg",
        },
        "4664c4dc-89a2-4dd0-9267-ab4ff65af757": {
            "@type": "image",
            "alt": "max-kukurudziak-MedRVIJX2P0-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/max-kukurudziak-medrvijx2p0-unsplash.jpg",
        },
        "4b07582a-771a-4ce9-92bf-456e33134dd3": {
            "@type": "image",
            "alt": "daniel-cheung-cPF2nlWcMY4-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/daniel-cheung-cpf2nlwcmy4-unsplash.jpg",
        },
        "5c4e9474-0ccb-45a4-a91d-5eba411e43ef": {
            "@type": "text",
            "text": {
                "blocks": [
                    {
                        "data": {},
                        "depth": 0,
                        "entityRanges": [],
                        "inlineStyleRanges": [],
                        "key": "98gs2",
                        "text": "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. ",
                        "type": "unstyled",
                    }
                ],
                "entityMap": {},
            },
        },
        "6ab1e289-f46a-495f-b07d-6967c436e369": {
            "@type": "text",
            "text": {
                "blocks": [
                    {
                        "data": {},
                        "depth": 0,
                        "entityRanges": [],
                        "inlineStyleRanges": [],
                        "key": "f0er1",
                        "text": "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. ",
                        "type": "unstyled",
                    }
                ],
                "entityMap": {},
            },
        },
        "7471143d-70da-493c-b3c5-56a021407bed": {"@type": "title"},
        "7550e1c3-03b9-432a-bdd1-2d2f33ef6d68": {
            "@type": "image",
            "alt": "clem-onojeghuo-RfAEjh_J6e0-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/clem-onojeghuo-rfaejh_j6e0-unsplash.jpg",
        },
        "828b9212-2b98-4192-92bd-ac6acc08b01f": {
            "@type": "image",
            "alt": "daniel-von-appen-tb4heMa-ZRo-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/daniel-von-appen-tb4hema-zro-unsplash.jpg",
        },
        "82b244cc-25fd-41c7-91bd-d8aea6540b61": {
            "@type": "image",
            "alt": "cristina-gottardi-Pl8ZMtFiyi8-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/cristina-gottardi-pl8zmtfiyi8-unsplash.jpg",
        },
        "976bea21-fe53-477f-b1a8-47fd51d88492": {
            "@type": "text",
            "text": {
                "blocks": [
                    {
                        "data": {},
                        "depth": 0,
                        "entityRanges": [],
                        "inlineStyleRanges": [],
                        "key": "3m6c5",
                        "text": "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. ",
                        "type": "unstyled",
                    }
                ],
                "entityMap": {},
            },
        },
        "9d1f39db-dfe5-4315-9306-1be2738e0068": {
            "@type": "text",
            "text": {
                "blocks": [
                    {
                        "data": {},
                        "depth": 0,
                        "entityRanges": [],
                        "inlineStyleRanges": [],
                        "key": "4q22n",
                        "text": "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. ",
                        "type": "unstyled",
                    }
                ],
                "entityMap": {},
            },
        },
        "a1fafe33-f85f-46fa-919a-19111a8bde52": {
            "@type": "image",
            "alt": "doug-linstedt-q07oPCdhPlw-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/doug-linstedt-q07opcdhplw-unsplash.jpg",
        },
        "c513539d-3b75-4f36-baa3-7145324f79ff": {
            "@type": "text",
            "text": {
                "blocks": [
                    {
                        "data": {},
                        "depth": 0,
                        "entityRanges": [],
                        "inlineStyleRanges": [],
                        "key": "1l2bg",
                        "text": "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. ",
                        "type": "unstyled",
                    }
                ],
                "entityMap": {},
            },
        },
        "cce67a5b-3dae-4e78-9352-7ccb24bb16a5": {
            "@type": "image",
            "alt": "christian-regg-jOicNGJIwZA-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/christian-regg-joicngjiwza-unsplash.jpg",
        },
        "d43abdc7-29c4-4cae-b3ed-443c87c449cd": {
            "@type": "text",
            "text": {
                "blocks": [
                    {
                        "data": {},
                        "depth": 0,
                        "entityRanges": [],
                        "inlineStyleRanges": [],
                        "key": "8iog5",
                        "text": "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. ",
                        "type": "unstyled",
                    }
                ],
                "entityMap": {},
            },
        },
        "dc517784-5cfc-4cf8-8fad-15fd154cd43a": {
            "@type": "image",
            "alt": "john-barkiple-l090uFWoPaI-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/john-barkiple-l090ufwopai-unsplash.jpg",
        },
        "e44ec113-a62d-4913-89d0-ba8d7e50d248": {
            "@type": "image",
            "alt": "denny-muller-jLjfAWwHdB8-unsplash.jpg",
            "url": "http://localhost:8080/Plone/page/denny-muller-jljfawwhdb8-unsplash.jpg",
        },
    }
    volto_page.blocks_layout = {
        "items": [
            "7471143d-70da-493c-b3c5-56a021407bed",
            "7550e1c3-03b9-432a-bdd1-2d2f33ef6d68",
            "23dd130c-d90e-48d8-ab5a-9fb85a2136a4",
            "cce67a5b-3dae-4e78-9352-7ccb24bb16a5",
            "24b5b799-d755-4e43-97f5-e240d6b3b7df",
            "82b244cc-25fd-41c7-91bd-d8aea6540b61",
            "4b07582a-771a-4ce9-92bf-456e33134dd3",
            "828b9212-2b98-4192-92bd-ac6acc08b01f",
            "e44ec113-a62d-4913-89d0-ba8d7e50d248",
            "a1fafe33-f85f-46fa-919a-19111a8bde52",
            "367aacb2-8124-43ce-8989-c9394b71ba97",
            "2172b929-70de-416b-b345-b98916a9726f",
            "dc517784-5cfc-4cf8-8fad-15fd154cd43a",
            "38076104-d93a-4e25-b2b6-7917be41dd48",
            "23e99d38-da30-441b-8713-bbbad4344a85",
            "4664c4dc-89a2-4dd0-9267-ab4ff65af757",
            "374bcb3f-d6c7-4ec0-9a26-aa9ee148a22a",
            "c513539d-3b75-4f36-baa3-7145324f79ff",
            "28b1c856-4ecd-4c43-bce5-9be127f29a34",
            "6ab1e289-f46a-495f-b07d-6967c436e369",
            "2e005fdb-a109-4148-b78c-855dab1e83a9",
            "d43abdc7-29c4-4cae-b3ed-443c87c449cd",
            "5c4e9474-0ccb-45a4-a91d-5eba411e43ef",
            "976bea21-fe53-477f-b1a8-47fd51d88492",
            "9d1f39db-dfe5-4315-9306-1be2738e0068",
            "0d57ccc7-8ceb-4b8f-bf95-5919eb859925",
        ]
    }
    publish(volto_page)
    volto_page.reindexObject()

    # # add images
    # from os import listdir
    # from os.path import isfile, join

    # currentdir = os.path.dirname(__file__)
    # base_image_path = os.path.join(currentdir, "performance")
    # filenames = [
    #     f for f in listdir(base_image_path) if isfile(join(base_image_path, f))
    # ]
    # for filename in filenames:
    #     imagepath = os.path.join(base_image_path, filename)
    #     volto_page.invokeFactory("Image", id=filename, title=filename)
    #     import sys
    #     import pdb

    #     for attr in ("stdin", "stdout", "stderr"):
    #         setattr(sys, attr, getattr(sys, "__%s__" % attr))
    #     pdb.set_trace()
    #     portal.volto[filename].image = NamedBlobImage(
    #         data=open(filename, "r").read(),
    #         filename=imagepath,
    #         contentType="image/jpeg",
    #     )
